var targetPitch = 0; /* radians of orientation around the X axis */
var targetYaw = 0;   /* radians of orientation around the Y axis */
var targetRotation = 0;  /* radians of orientation around the Z axis */
var targetTranslation = 0;  /* distance in inches from Z axis */
var targetPanX = 0;  /* distance in inches from X axis */
var targetPanY = 0;  /* distance in inches from Y axis */
var targetRotationOnMouseDown = 0; /* Z axis */
var mouseX = 0;      /* pixels from left margin of visual canvas */
var mouseY = -1;     /* pixels from top margin of visual canvas */
var prevMouseX = -1;
var prevMouseY = -1;
var mouseXOnMouseDown = 0;
var windowHalfX = window.innerWidth / 2;  /* midpoint pixel from left to right */
var windowHalfY = window.innerHeight / 2; /* midpoint pixel from top to bottom */
var canvasWidth = 818 /* 818 | 606 | 740 */;
var canvasHeight = 523 /* 523 | 602 | 574 */;
var THREE = require( 'three' );
var scene, camera, raycaster; 
var group;       /* used as parent node for all visualization geometry */
var depth_proxy; /* used to calculate ceiling components depth only */
var renderer = new THREE.WebGLRenderer( { preserveDrawingBuffer: true, antialias: true } );
var meshes_built = []; /* array of meshes built to be used for memory disposal upon update */
var max_x = 0; var max_z = 0;         /* used to position camera */
var min_x = 99999; var min_z = 99999; /* used to position camera */
var geocenter_x = 0.0;  /* used to position geometry center for rotation pivot */
var geocenter_y = 0.0;  /* used to position geometry center for rotation pivot */
var geocenter_z = 0.0;  /* used to position geometry center for rotation pivot */
var scene_height = 100; /* used to position camear */
var objects = [];       /* geometry array of those to check for picking */
var reset_sprite;       /* The four GUI images for user interaction with scene */
var threed_sprite;
var zoomin_sprite;
var zoomout_sprite;
var snapshot = "";      /* Image string for latest canvas snapshot to image */
var bevelAmt = .25;     /* distance in inches for interbox bevel */
var btnCode = -1;       /* integer enumerator for current mouse button being used */
var featureType = 0;    /* 0:StraightPartition, 1:ArcPartition, 2:Facing, 3:Transition, 4:Ceiling, 5:BentPartition */
var featureSize = -1;   /* x, y, z distance vector for size of feature geometry */
var ceilingRun = -1;	/* the length of the ceiling run of a wall-ceiling feature */
var material = 0xffffff; /* base color for geometry as hexidecimal */
var test = false;        /* boolean for test mode or production */
var leftBox = [];        /* vertices locations for leftmost Box within feature */
var rightBox = [];       /* vertices locations for rightmost Box within feature */
var lighting = "";   /* specialized lighting type as string */

/* COLOR NOTES FOR SEEYOND FEATURES:
 * Black C = 0x2D2926 (ebony)
 * 2334 C = 0x6C6463 (cast)
 * 427 C = 0xD0D3D4 (nickel)
 * Warm Gray 1 C = 0xD7D2CB (zinc)
 * Cool Gray 11 C = 0x53565A (dark_gray)
 * 418 C = 0x51534A (ore)
 * 7529 C = 0xB7A99A (cashmere)
 * 7589 C = 0x5C4738 (burnt umber)
*/

/*
* Top level export object for NPM module use
*/
(function(exports) {

    /*
    * The primary Visualization services. This will have all functionality required to support the current product line.
    * QT stands for quad tessellator as these tessellations all have 4 sides. By using the QT namespace, we limit this
    * development to four-sided patterns. This module provides visualization services for all geometry produced by the
    * syd-tessellation module or produced by syd-tiling module.
    */

    exports.QT = {
        Visualization: {
            /*
            * added as simple way to confirm module is loaded
            */
            Loaded: function() {
                return "syd-visualization is loaded";
            },

            RunInTestMode: function() {
                test = true;
            },

            SetCanvasSize: function(width, height) {
                canvasWidth = width;
                canvasHeight = height;
            },
			/* Used to visualize a set of tiles from syd-tiling module */
            visualizeTiles: function(uNum, vNum, boxes, gridData, height) {
            	console.log(boxes);
            	scene_height = height;
            	var feature = new Syd_QT_Feature(boxes);
            	this.visualizeScene(uNum, vNum, "multiple", gridData, "front", feature, height);
            },
			/* Used to visualize a feature generated from syd-tessellation module */
			visualizeFeature: function(front, back, uNum, vNum, mat, surf, lit) {
				var boxes = GenerateBoxesFromSurfaces(front, back, uNum, vNum);
				var feature = new Syd_QT_Feature(boxes);
				mat = new Array(mat);
				var matindx = new Array(0);			
				lighting = lit;
            	this.visualizeScene(uNum, vNum, mat, "no grid", surf, feature, -1);
			},
			/* Downstream visualization services for both tiles features and tessellated features */
            visualizeScene: function(uNum, vNum, mat, gridData, surf, feature, height) {
                if(test == true) {
                    var t0 = performance.now(); //use this for performance timing profiles
                }
                //console.log("Visualizing the Feature");
                group = new THREE.Group();
                depth_proxy = new THREE.Group();
                
                material = mat[0];
				/* conversion services for limited set of named colors */
                switch(mat[0]){
                    case 'burnt_umber':
                        material = 0x5c4738;
                    break;

                    // arc partition
                    case 'cashmere':
                        material = 0xb7a99a;
                    break;

                    // straight facing
                    case 'cast':
                        material = 0x6c6463;
                    break;

                    //transition
                    case 'dark_gray':
                        material = 0x53565a;
                    break;

                    //ceiling
                    case 'ebony':
                        material = 0x2d2926;
                    break;

                    //bent partition
                    case 'nickel':
                        material = 0xd0d3d4;
                    break;

                    //bent partition
                    case 'ore':
                        material = 0x51534a;
                    break;

                    //bent partition
                    case 'zinc':
                        material = 0xd7d2cb;
                    break;
				}

                init(material);  /* initialize the visualization */
                animate(); /* animate visualization for changes through user interaction */

                function init(material) {
                  if(meshes_built.length > 0) {
                      for(i=0;i<meshes_built.length;i++) {
                          scene.remove(meshes_built[i]);
                          //Not needed since November 2015 (due to update in three.js services):
                          //meshes_built[i].dispose();
                      }
                      /* clean-up conservatively for any previous action on the canvas */
                      scene.remove(camera);
                      scene.remove(light);
                      scene.remove(group);
                      scene.remove(reset_sprite);
                      scene.remove(threed_sprite);
                      scene.remove(zoomin_sprite);
                      scene.remove(zoomout_sprite);
                      objects = [];
                      scene = null;
                  }
                  /* three.js requires a scene element as root to visualization */
                  scene = new THREE.Scene();
                  /* three.js camera defines point-of-view for visualization */
                  camera = new THREE.PerspectiveCamera( 90, canvasWidth/canvasHeight, 1, 10000 );
                  scene.add( camera );
				  if(lighting == "cove") {
					light = new THREE.AmbientLight( 0xffffff, .5 );
					//light = new THREE.DirectionalLight( 0xffffff, .5 );
                    //light.position.set(0, 0, -1);	
				  } else {  
					  if(featureType == 0 || featureType == 1 || featureType == 500) {
						light = new THREE.PointLight( 0xffffff, 0.5 );
						camera.add( light );
						light = new THREE.AmbientLight( 0xffffff, 0.5 );
					  } else {
						light = new THREE.PointLight( 0xffffff, 0.8 );
					  }
				  }
				  camera.add( light );
                  group.position.y = 0;
                  scene.add( group );
                  meshes_built = [];
                  /* textures give geometry materials the look of a material type (e.g. felt) */
                  var texture = [];
                  var loader = new THREE.TextureLoader();

                  if(featureType < 100) { //One of the Seeyond Products
                  	texture.push(loader.load( material ));
                  	texture[0].wrapS = texture[0].wrapT = THREE.RepeatWrapping;
                  	/* defines the size of the texture use on geometry */
                  	texture[0].repeat.set( .03, .03 );
                  } else {
                  	//Make multiple textures when the feature allows multiple (per tile for example)
                  	var material = [];
                  	var matindx = [];

					if(featureType == 300 || featureType == 400 || featureType == 500) { //Velo, HushBlock, and Swoon products are not a grid
						uNum = gridData.length;
						vNum = 1;
						var mat_count = 0;
						for(var m=0;m<uNum;m++) {
							material.push(gridData[m].texture);
							matindx.push(mat_count++);
						}
					} else {
						var mat_count = 0;
						for(var m=0;m<vNum;m++) {
							for(var n=0;n<uNum;n++) {
								material.push(gridData[m][n].texture);
								matindx.push(mat_count++);
							}
						}
					}
                  
                  	for(t=0;t<material.length;t++) {
                  		texture.push(loader.load( material[t] ))
                  		texture[t].wrapS = texture.wrapT = THREE.RepeatWrapping;
                  		texture[t].repeat.set( 1, 1 );
                  	}
                  }

                  //Find the geometric center of the feature
                  if(featureType < 100) { //Seeyond Product
                      /* syd-tessellation passes tessellated boxes as an array of vertices */
                  	  var cb_flts = feature.getBox(Math.floor(uNum*vNum/2)).getFloats();
                  	  geocenter_x = (cb_flts[0] + cb_flts[6])/2.0;
                  	  geocenter_y = (cb_flts[1] + cb_flts[7])/2.0;
                  	  geocenter_z = (cb_flts[2] + cb_flts[8])/2.0;
				  } else if(featureType == 300) { //Velo
				  	  for (var i=0; i<(uNum*vNum); i++) {
						  var row = feature.getBox(i).getFloats()[15];
						  var column = feature.getBox(i).getFloats()[17];
						  var upperleft_x = column * (47.82);
						  var upperleft_z = row * (23.92);
						  if(row % 2 == 0) {
							  upperleft_x -= 23.92;
						  }
						  if(upperleft_x + 32.0 > max_x) {
							  max_x = upperleft_x + 32.0;
						  }
						  if(upperleft_x < min_x) {
							  min_x = upperleft_x;
						  }
						  if(upperleft_z + 20.0 > max_z) {
							  max_z = upperleft_z+ 20.0;
						  }		
						  if(upperleft_z < min_z) {
							  min_z = upperleft_z;
						  }
					  }
                  	  geocenter_x = (min_x + max_x)/2.0;
                  	  geocenter_z = (min_z + max_z)/2.0;
				  } else if(featureType == 400) { //HushBlock
				  	  if(uNum > 0) {
				  	  	geocenter_x = (24.0 * feature.getBox(0).getFloats()[18])/2.0;
                  	  	geocenter_y = (24.0 * feature.getBox(0).getFloats()[20])/2.0;
                  	  	geocenter_z = 1.0;
                  	  	//alert('Geocenter: ' + geocenter_x + "," + geocenter_y);
                  	  }
				  } else if(featureType == 500) { //Swoon
				  	  for (var i=0; i<(uNum*vNum); i++) {
						  var row = feature.getBox(i).getFloats()[15];
						  var column = feature.getBox(i).getFloats()[17];
						  
						  for (var j=0; j<4; j++) {
							  if(feature.getBox(i).getFloats()[j*3+0] > max_x) {
								  max_x = feature.getBox(i).getFloats()[j*3+0];
							  }
							  if(feature.getBox(i).getFloats()[j*3+0] < min_x) {
								  min_x = feature.getBox(i).getFloats()[j*3+0];
							  }
							  if(feature.getBox(i).getFloats()[j*3+2] > max_z) {
								  max_z = feature.getBox(i).getFloats()[j*3+2];
							  }
							  if(feature.getBox(i).getFloats()[j*3+2] < min_z) {
								  min_z = feature.getBox(i).getFloats()[j*3+2];
							  }
						  }
					  }
                  	  geocenter_x = (min_x + max_x)/2.0;
                  	  geocenter_z = (min_z + max_z)/2.0;
                  } else {
                  	geocenter_x = (24.0 * uNum)/2.0;
                  	geocenter_z = (24.0 * vNum)/2.0;
                  }
				  if(featureType != 500) {
					max_x = 0;
					max_z = 0;
					min_x = 99999;
					min_z = 99999;
				  }
				  //march through all tiles or boxes passed in from syd-tessellation or syd-tiling
				  for (var i=0; i<(uNum*vNum); i++) {
					  if(featureType == 200 && feature.getBox(i).getFloats()[2] == -10000.0 && feature.getBox(i).getFloats()[0] != -10000.0) {
				      	  var geometry = new THREE.Geometry();
				      	  var flts = feature.getBox(i).getFloats();
				      	  if(flts[1] == 90 || flts[1] == 270) {
						  	geometry = GenerateClarioTile(geometry, 4.75, i%uNum, Math.floor(i/uNum), 24, flts[0], flts[1]);
						  } else {
						  	geometry = GenerateClarioTile(geometry, 4.75, i%uNum, Math.floor(i/uNum), flts[0], 24, flts[1]);
						  }
				      } else if(featureType == 300) {
				      	  var geometry = new THREE.Geometry();
				      	  if(feature.getBox(i).getFloats()[1] == 1) {
				      	  	geometry = GenerateVeloTile(geometry, feature, i, -2.77);
				      	  } else {
				      	  	geometry = GenerateVeloTile(geometry, feature, i, 2.77);
				      	  }
				      } else if(featureType == 400) {
				      	  var geometry = new THREE.Geometry();
				      	  geometry = GenerateHushBlockTile(geometry, feature, i, 2.0);
				      } else if(featureType == 500) {
				      	  var geometry = new THREE.Geometry();
				      	  if(feature.getBox(i).getFloats()[1] == 1) {
				      	  	geometry = GenerateSwoonTile(geometry, feature, i, -2.77);
				      	  } else {
				      	  	geometry = GenerateSwoonTile(geometry, feature, i, 2.77);
				      	  }
                      } else {
						  var geometry = new THREE.Geometry();
						  var flts = feature.getBox(i).getFloats();

						  if(i==0) {
							  leftBox = flts;
						  } else if (i==(uNum*vNum-vNum)) {
							  rightBox = flts;
						  }

						  var front_points = new THREE.Geometry();
						  var back_points = new THREE.Geometry();

						  var front_points_b = new THREE.Geometry();
						  var back_points_f = new THREE.Geometry();
						  //add bevels in visualization between boxes or tiles
						  for(var g=0; g<12; g=g+3 ){
							  //front, offset in for chamfer
							  if(featureType == 3 || featureType == 5) {
								  var frnt = new THREE.Vector3 (flts[g] - geocenter_x, flts[g+1], flts[g+2] - geocenter_z);
								  var bck = new THREE.Vector3 (flts[12+g] - geocenter_x, flts[12+g+1], flts[12+g+2] - geocenter_z);
							  } else {
								  var frnt = new THREE.Vector3 (flts[g] - geocenter_x, flts[g+1] - geocenter_y, flts[g+2] - geocenter_z);
								  var bck = new THREE.Vector3 (flts[12+g] - geocenter_x, flts[12+g+1] - geocenter_y, flts[12+g+2] - geocenter_z);                                
							  } 
							  var vec = new THREE.Vector3 ( bck.x - frnt.x, bck.y - frnt.y, bck.z - frnt.z );
							  if(featureType>99) {
								  bevelAmt = 0.0;
							  }
							  vec.setLength(bevelAmt);
							  var bck_f = new THREE.Vector3 ( bck.x - vec.x, bck.y - vec.y, bck.z - vec.z ); 
							  var frnt_b = new THREE.Vector3 ( frnt.x + vec.x, frnt.y + vec.y, frnt.z + vec.z ); 

							  front_points.vertices.push( frnt );
							  front_points_b.vertices.push( frnt_b );
							  back_points.vertices.push( bck );  
							  back_points_f.vertices.push( bck_f );
						  }

						  for(var h=0; h<4; h++ ){
							  var last = h - 1;
							  if(last < 0) {
								  last = 4 + last;
							  }
							  var next = h + 1;
							  if(next > 3) {
								  next = 0; 
							  }

							  //inset fronts
							  var front_vec1 = new THREE.Vector3( front_points.vertices[last].x - front_points.vertices[h].x,
																  front_points.vertices[last].y - front_points.vertices[h].y,
																  front_points.vertices[last].z - front_points.vertices[h].z  );

							  var front_vec2 = new THREE.Vector3( front_points.vertices[next].x - front_points.vertices[h].x,
																  front_points.vertices[next].y - front_points.vertices[h].y,
																  front_points.vertices[next].z - front_points.vertices[h].z );
					
							  var front_angle = front_vec1.angleTo(front_vec2);
							  var vector_length = bevelAmt / Math.sin(front_angle);
							  front_vec1.setLength(vector_length);
							  front_vec2.setLength(vector_length);
							  front_points.vertices[h].add(front_vec1).add(front_vec2);

							  //inset backs
							  var back_vec1 = new THREE.Vector3( back_points.vertices[last].x - back_points.vertices[h].x,
															  back_points.vertices[last].y - back_points.vertices[h].y, 
															  back_points.vertices[last].z - back_points.vertices[h].z );
							  var back_vec2 = new THREE.Vector3( back_points.vertices[next].x - back_points.vertices[h].x, 
															  back_points.vertices[next].y - back_points.vertices[h].y, 
															  back_points.vertices[next].z - back_points.vertices[h].z );

							  var back_angle = back_vec1.angleTo(back_vec2);
							  vector_length = bevelAmt / Math.sin(back_angle);
							  back_vec1.setLength(vector_length);
							  back_vec2.setLength(vector_length);
							  back_points.vertices[h].add(back_vec1).add(back_vec2);
						  }

						  //combine into single Array
						  geometry.vertices = front_points.vertices.concat( back_points.vertices,
																			front_points_b.vertices,
																			back_points_f.vertices);
						  geometry.faces.push( new THREE.Face3( 2, 1, 0 ) ); // front
						  geometry.faces.push( new THREE.Face3( 3, 2, 0 ) );
						  geometry.faces.push( new THREE.Face3( 4, 5, 6 ) ); // back
						  geometry.faces.push( new THREE.Face3( 4, 6, 7) );
				
						  if(featureType < 100) {                   
							  //front bevel
							  geometry.faces.push( new THREE.Face3( 0, 1, 8 ) ); // top
							  geometry.faces.push( new THREE.Face3( 1, 9, 8) );
							  geometry.faces.push( new THREE.Face3( 1, 2, 9 ) ); // right
							  geometry.faces.push( new THREE.Face3( 2, 10, 9) );
							  geometry.faces.push( new THREE.Face3( 2, 3, 10 ) ); // bottom
							  geometry.faces.push( new THREE.Face3( 3, 11, 10) );
							  geometry.faces.push( new THREE.Face3( 3, 0, 11 ) ); // left
							  geometry.faces.push( new THREE.Face3( 0, 8, 11) );
				
							  //back bevel
							  geometry.faces.push( new THREE.Face3( 12, 5, 4 ) ); // top
							  geometry.faces.push( new THREE.Face3( 12, 13, 5) );
							  geometry.faces.push( new THREE.Face3( 13, 6, 5 ) ); // right
							  geometry.faces.push( new THREE.Face3( 13, 14, 6) );
							  geometry.faces.push( new THREE.Face3( 14, 7, 6 ) ); // bottom
							  geometry.faces.push( new THREE.Face3( 14, 15, 7) );
							  geometry.faces.push( new THREE.Face3( 15, 4, 7 ) ); // left
							  geometry.faces.push( new THREE.Face3( 15, 12, 4) );
						  }
						  var direction = 1;
						  if(featureType==1) {
							  direction = -1; //Arc Partitions boxes drawn in reverse order
						  }
						  //middle
						  if(i<vNum || featureType > 99) { //left
							  if((featureType == 0 || featureType == 1)) {
								if(i>0 && i<vNum-1) {
									GenerateMidSeamFromBoxes(geometry, 8, 9, 12, 13, 2, "mdl", bevelAmt);

									geometry.faces.push( new THREE.Face3( 8, 9, 16 ) ); // left with bevel
									geometry.faces.push( new THREE.Face3( 9, 18, 16 ) );
									geometry.faces.push( new THREE.Face3( 17, 19, 12 ) );
									geometry.faces.push( new THREE.Face3( 19, 13, 12) );
									
									geometry.faces.push( new THREE.Face3( 20, 18, 16 ) );
									geometry.faces.push( new THREE.Face3( 21, 20, 18 ) );
									geometry.faces.push( new THREE.Face3( 20, 19, 17 ) );
									geometry.faces.push( new THREE.Face3( 21, 20, 19) );
								}
							  } else {
								geometry.faces.push( new THREE.Face3( 8, 9, 12 ) ); // left no bevel
								geometry.faces.push( new THREE.Face3( 9, 13, 12) );
							  }
						  }
						  if(i>=(uNum*vNum)-vNum || featureType > 99) { //right
							  if((featureType == 0 || featureType == 1)) {
								if(i>(uNum*vNum)-vNum && i<(uNum*vNum)-1) {
									GenerateMidSeamFromBoxes(geometry, 10, 11, 14, 15, 0, "mdl", bevelAmt);

									geometry.faces.push( new THREE.Face3( 10, 11, 16 ) ); // right with bevel
									geometry.faces.push( new THREE.Face3( 11, 18, 16) );
									geometry.faces.push( new THREE.Face3( 17, 19, 14 ) );
									geometry.faces.push( new THREE.Face3( 19, 15, 14) );

									geometry.faces.push( new THREE.Face3( 20, 18, 16 ) );
									geometry.faces.push( new THREE.Face3( 21, 20, 18 ) );
									geometry.faces.push( new THREE.Face3( 20, 19, 17 ) );
									geometry.faces.push( new THREE.Face3( 21, 20, 19) );
								}								
							  } else {
								geometry.faces.push( new THREE.Face3( 10, 11, 14 ) ); // right no bevel
								geometry.faces.push( new THREE.Face3( 11, 15, 14) );
							  }
						  }
						  if(i%vNum==(vNum-1) || featureType > 99) {
							  if((featureType == 0 || featureType == 1)) { //top
								GenerateMidSeamFromBoxes(geometry, 9, 10, 13, 14, 0, "mdl", bevelAmt);

								geometry.faces.push( new THREE.Face3( 9, 10, 16 ) ); // top with bevel
								geometry.faces.push( new THREE.Face3( 10, 18, 16) );
								geometry.faces.push( new THREE.Face3( 17, 19, 13 ) );
								geometry.faces.push( new THREE.Face3( 19, 14, 13) );

								geometry.faces.push( new THREE.Face3( 20, 18, 16 ) );
								geometry.faces.push( new THREE.Face3( 21, 20, 18) );
								geometry.faces.push( new THREE.Face3( 20, 19, 17 ) );
								geometry.faces.push( new THREE.Face3( 21, 20, 19) );

								if(i==0 || i==(vNum-1)) {
									GenerateMidSeamFromBoxes(geometry, 8, 9, 12, 13, 2, "ltc", bevelAmt);

									geometry.faces.push( new THREE.Face3( 8, 9, 22 ) ); // left with bevel
									geometry.faces.push( new THREE.Face3( 9, 24, 22 ) );
									geometry.faces.push( new THREE.Face3( 23, 25, 12 ) );
									geometry.faces.push( new THREE.Face3( 25, 13, 12) );

									geometry.faces.push( new THREE.Face3( 26, 24, 22 ) );
									geometry.faces.push( new THREE.Face3( 27, 26, 24 ) );
									geometry.faces.push( new THREE.Face3( 26, 25, 23 ) );
									geometry.faces.push( new THREE.Face3( 27, 26, 25) );	
								}
								if(i==(uNum*vNum)-vNum || i==(uNum*vNum)-1) {
									GenerateMidSeamFromBoxes(geometry, 10, 11, 14, 15, 0, "rtc", bevelAmt);

									geometry.faces.push( new THREE.Face3( 10, 11, 22 ) ); // right with bevel
									geometry.faces.push( new THREE.Face3( 11, 24, 22 ) );
									geometry.faces.push( new THREE.Face3( 23, 25, 14 ) );
									geometry.faces.push( new THREE.Face3( 25, 15, 14) );

									geometry.faces.push( new THREE.Face3( 26, 24, 22 ) );
									geometry.faces.push( new THREE.Face3( 27, 26, 24 ) );
									geometry.faces.push( new THREE.Face3( 26, 25, 23 ) );
									geometry.faces.push( new THREE.Face3( 27, 26, 25) );
								}
							  } else {
								geometry.faces.push( new THREE.Face3( 9, 10, 13 ) ); // top no bevel
								geometry.faces.push( new THREE.Face3( 10, 14, 13) );
							  }
						  }
						  if(i%vNum==0 || featureType > 99) {
							  if((featureType == 0 || featureType == 1)) { //bottom
								GenerateMidSeamFromBoxes(geometry, 8, 11, 12, 15, 1, "mdl", bevelAmt);
							    
								geometry.faces.push( new THREE.Face3( 8, 11, 16 ) ); // bottom with bevel
							    geometry.faces.push( new THREE.Face3( 11, 18, 16 ) );
								geometry.faces.push( new THREE.Face3( 17, 19, 12 ) );
							    geometry.faces.push( new THREE.Face3( 19, 15, 12 ) );

								geometry.faces.push( new THREE.Face3( 20, 18, 16 ) );
								geometry.faces.push( new THREE.Face3( 21, 20, 18) );
								geometry.faces.push( new THREE.Face3( 20, 19, 17 ) );
								geometry.faces.push( new THREE.Face3( 21, 20, 19) );
							  	if(i==0 || i==(vNum-1)) {
									GenerateMidSeamFromBoxes(geometry, 8, 9, 12, 13, 2, "lbc", bevelAmt);

									geometry.faces.push( new THREE.Face3( 8, 9, 22 ) ); // left with bevel
									geometry.faces.push( new THREE.Face3( 9, 24, 22 ) );
									geometry.faces.push( new THREE.Face3( 23, 25, 12 ) );
									geometry.faces.push( new THREE.Face3( 25, 13, 12) );

									geometry.faces.push( new THREE.Face3( 26, 24, 22 ) );
									geometry.faces.push( new THREE.Face3( 27, 26, 24 ) );
									geometry.faces.push( new THREE.Face3( 26, 25, 23 ) );
									geometry.faces.push( new THREE.Face3( 27, 26, 25) );							
								}
								if(i==(uNum*vNum)-vNum || i==(uNum*vNum)-1) {
									GenerateMidSeamFromBoxes(geometry, 10, 11, 14, 15, 0, "rbc", bevelAmt);

									geometry.faces.push( new THREE.Face3( 10, 11, 22 ) ); // right with bevel
									geometry.faces.push( new THREE.Face3( 11, 24, 22 ) );
									geometry.faces.push( new THREE.Face3( 23, 25, 14 ) );
									geometry.faces.push( new THREE.Face3( 25, 15, 14) );

									geometry.faces.push( new THREE.Face3( 26, 24, 22 ) );
									geometry.faces.push( new THREE.Face3( 27, 26, 24 ) );
									geometry.faces.push( new THREE.Face3( 26, 25, 23 ) );
									geometry.faces.push( new THREE.Face3( 27, 26, 25) );
								}								
							  } else {
							    geometry.faces.push( new THREE.Face3( 11, 8, 15 ) ); // bottom no bevel
							    geometry.faces.push( new THREE.Face3( 8, 12, 15) );
							  }
						  }
						  //As of November 2015, we need to flush webGL memory (see destroyFeature method):
						  front_points.dispose();
						  back_points.dispose();
						  front_points_b.dispose();
						  back_points_f.dispose();
					  }
					  geometry.computeFaceNormals();
					  geometry.computeVertexNormals();
					  //UVs determine how a texture is applied to each face of geometry
					  assignUVs(geometry);

					  if(typeof material == 'string') {
						  var td_material = new THREE.MeshPhongMaterial( { side: THREE.DoubleSide, map: texture[0] } );
					  } else if(typeof material[0] == 'string') {
						  var td_material = new THREE.MeshPhongMaterial( { side: THREE.DoubleSide, map: texture[matindx[i]] } );
					  } else {
						  var td_material = new THREE.MeshPhongMaterial( { color: material, side: THREE.DoubleSide } );
					  }

					  meshes_built.push(new THREE.Mesh( geometry, td_material ));
					  group.add( meshes_built[i] );
					  if(featureType==3) {
					  	var up = Math.abs(flts[14] - flts[2]);
					  	var back = Math.abs(flts[13] - flts[1]);
					  	//if a ceiling box (not a wall box) then use geometry services to determine the depth of the feature
					  	if(Math.abs(flts[14] - flts[2]) > Math.abs(flts[13] - flts[1])) {
					  		depth_proxy.add(new THREE.Mesh( geometry, td_material ));
					  	}
					  }
					  //As of November 2015, we need to flush webGL memory (see destroyFeature method):
					  geometry.dispose();
					  td_material.dispose();
				  } //end of box/tile generation

				  if(featureType < 100) {
					  texture[0].dispose();
				  } else {
					  for(t=0; t<texture.length; t++) {
						  texture[t].dispose();
					  }
				  }				  
				  //features from tessellation need an extra step to be oriented parallel to the X-axis
				  if(featureType < 100) {
                  	OrientX(surf, geocenter_x, geocenter_y , geocenter_z);
				  }
				  //bounding box service is useful for getting metrics regarding size of feature
                  var boundingBox = new THREE.Box3();
                  featureSize = new THREE.Vector3();
                  var helper = new THREE.BoxHelper(group, 0xff0000);
                  if(featureType == 3) {
					boundingBox.setFromObject(helper);
                    boundingBox.getSize(featureSize);
					ceilingRun = featureSize.z;
					helper = new THREE.BoxHelper(depth_proxy, 0xff0000);
                  }
                  boundingBox.setFromObject(helper);
                  boundingBox.getSize(featureSize);
                  //Uncomment to see the bounding box:
                  //scene.add(helper);

                  if(featureType == 300) { //Velo
                  	featureSize.x = max_x - min_x;
                  	featureSize.y = 12;
                  	featureSize.z = max_z - min_z;
                  } else if(featureType == 400) { //HushBlock
                    //alert(min_x + "," + max_x + " | " + min_z + "," + max_z);
                  	featureSize.x = max_x - min_x;
                  	featureSize.y = max_z - min_z;
                  	featureSize.z = 2;
                  } else if(featureType == 500) { //Swoon
                  	featureSize.x = max_x - min_x;
                  	featureSize.y = 1;
                  	featureSize.z = max_z - min_z;
                  } else if(featureType > 99) { //Clario or Tetria
                    featureSize.x = (24.0 * uNum);
                  	featureSize.y = height;
                  	featureSize.z = (24.0 * vNum);
                  } else {
                  	height = featureSize.y;
                  }

				  console.log("Feature Width: " + featureSize.x + "\n" +
							  "Feature Height: " + featureSize.y + "\n" + 
							  "Feature Depth: " + featureSize.z + "\n");

                  if(featureType == 300) { //Velo
                      camera.position.set(0.0, -scene_height/2, featureSize.x*1.2 );
                      camera.lookAt(new THREE.Vector3(0.0, -1.0*scene_height/2, 0.0));
                  } else if(featureType == 400) { //HushBlock
                  	  if(featureSize.x > featureSize.y) {
                      	camera.position.set(0.0, 50+featureSize.x/2, 0.0 );
                      } else {
                      	camera.position.set(0.0, 50+featureSize.y/2, 0.0 );                      
                      }
                      camera.lookAt(new THREE.Vector3(0.0, 0.0, 0.0));
				  } else if(featureType == 500) { //Swoon
                      camera.position.set(0.0, featureSize.y/2, featureSize.z );
                      camera.lookAt(new THREE.Vector3(0.0, featureSize.y/2, 0.0));
                  } else if(featureType == 3 || featureType == 5) {
                      if(featureSize.y > featureSize.x) {
                         camera.position.set( 0.0, featureSize.y/2, featureSize.y*1.4 );
                      } else {
                         camera.position.set( 0.0, featureSize.y/2, featureSize.x*1.7 );
                      }
                      camera.lookAt(new THREE.Vector3(0.0,featureSize.y/2,0.0));
                  } else { //Seeyond, Clario, Tetria
                      if(featureSize.y > featureSize.x) {
                          camera.position.set( 0.0, 1.85, featureSize.y*1.2 );
                      } else {
                          camera.position.set( 0.0, 1.85, featureSize.x*1.2 );
                      }
                      camera.lookAt(new THREE.Vector3(0.0, 0.0, 0.0));
                  }
                  //Set the initial orientation for feature type
                  initializeFeatureOrientation(height);
                  //Build additional geometry for feature's environment
                  if(featureType < 2) {
                    //Render a Ground Plane
                    geometry = new THREE.Geometry();
                    geometry.vertices.push( new THREE.Vector3 (1500, -geocenter_y, 1500) );
                    geometry.vertices.push( new THREE.Vector3 (1500, -geocenter_y, -1500) );
                    geometry.vertices.push( new THREE.Vector3 (-1500, -geocenter_y, -1500) );
                    geometry.vertices.push( new THREE.Vector3 (-1500, -geocenter_y, 1500) );
                    geometry.faces.push( new THREE.Face3( 0, 1, 2 ) ); // front
                    geometry.faces.push( new THREE.Face3( 0, 2, 3 ) );
                    geometry.computeVertexNormals();
                    meshes_built.push(new THREE.Mesh( geometry, new THREE.MeshPhongMaterial( { color: 0x808080, side: THREE.DoubleSide, transparent: true, opacity: 0.05 } ) ));
                    group.add( meshes_built[meshes_built.length-1] );
                    geometry.dispose();
                  } else if(featureType == 2 || featureType == 4) {
                    //Render a Wall Plane
                    var geometry = new THREE.Geometry();
					var wall_distance = 0;
					var wall_color = 0xffffff;
					var wall_opacity = 0.10;
				    if(lighting == "cove") {
						wall_distance = -15;
						wall_color = 0xffffff;
						wall_opacity = 0.25;
					}
					if(lighting == "cove" && featureType == 4) { //Ceilings only
						for(u=0; u<vNum; u++) {
							light = new THREE.PointLight( 0xffffff, /*intensity*/ 0.3, /*distance*/ 40, /*decay*/ 0.8 );
							light.position.set( -geocenter_x, -geocenter_y + u*(featureSize.y/vNum), -10 );
							group.add( light );
							light = new THREE.PointLight( 0xffffff, /*intensity*/ 0.3, /*distance*/ 40, /*decay*/ 0.8 );
							light.position.set( -geocenter_x+featureSize.x, -geocenter_y + u*(featureSize.y/vNum), -10 );
							group.add( light );
						}
						for(v=0; v<uNum+1; v++) {
							light = new THREE.PointLight( 0xffffff, /*intensity*/ 0.3, /*distance*/ 40, /*decay*/ 0.8 );
							light.position.set( -geocenter_x + v*(featureSize.x/uNum), -geocenter_y, -10 );
							group.add( light );
							light = new THREE.PointLight( 0xffffff, /*intensity*/ 0.3, /*distance*/ 40, /*decay*/ 0.8 );
							light.position.set( -geocenter_x + v*(featureSize.x/uNum), -geocenter_y+featureSize.y, -10 );
							group.add( light );
						}
						geoFloor = new THREE.BoxGeometry( featureSize.x, featureSize.y, 10 );
						var lightMaterial = new THREE.MeshPhongMaterial( { ambient: 0xffffff, color: 0xffffff, emissive: 0xaaaaaa, shininess: 0, transparent: true, opacity: 0.25 } );
						mshStdFloor = new THREE.Mesh( geoFloor, lightMaterial );
						mshStdFloor.position.set( featureSize.x/2-geocenter_x, featureSize.y/2-geocenter_y, -10 );
						group.add( mshStdFloor );
				 	}
					geometry.vertices.push( new THREE.Vector3 (1500, 1500, wall_distance) );
					geometry.vertices.push( new THREE.Vector3 (1500, -1500, wall_distance) );
					geometry.vertices.push( new THREE.Vector3 (-1500, -1500, wall_distance) );
					geometry.vertices.push( new THREE.Vector3 (-1500, 1500, wall_distance) );
					geometry.faces.push( new THREE.Face3( 0, 1, 2 ) ); // front
					geometry.faces.push( new THREE.Face3( 0, 2, 3 ) );
					geometry.computeVertexNormals();
					meshes_built.push(new THREE.Mesh( geometry, new THREE.MeshPhongMaterial( { color: wall_color, side: THREE.DoubleSide, transparent: true, opacity: wall_opacity } ) ));
					group.add( meshes_built[meshes_built.length-1] );
					geometry.dispose();
                  } else if(featureType == 3 || featureType == 5) { // Wall and Wall-Ceiling features
                    //Render a Wall Plane
                    var geometry = new THREE.Geometry();
					var wall_distance = 0;
					var wall_color = 0xffffff;
					var wall_opacity = 0.10;
				    if(lighting == "cove") {
						ceiling_distance = 10;
						wall_color = 0xffffff;
						wall_opacity = 0.25;
					}
					if(lighting == "cove" && featureType == 3) { //Ceilings only
						var numC = Math.floor(ceilingRun/18);
						for(u=0; u<numC; u++) {
							light = new THREE.PointLight( 0xffffff, /*intensity*/ 0.3, /*distance*/ 40, /*decay*/ 0.8 );
							light.position.set( -geocenter_x, 6,  + u*(ceilingRun/numC) );		
							group.add( light );
							light = new THREE.PointLight( 0xffffff, /*intensity*/ 0.3, /*distance*/ 40, /*decay*/ 0.8 );
							light.position.set( -geocenter_x+featureSize.x, 6, + u*(ceilingRun/numC) );
							group.add( light );
						}
						for(v=0; v<uNum+1; v++) {
							light = new THREE.PointLight( 0xffffff, /*intensity*/ 0.3, /*distance*/ 40, /*decay*/ 0.8 );
							light.position.set( -geocenter_x + v*(featureSize.x/uNum), 6, 0 );
							group.add( light );
							light = new THREE.PointLight( 0xffffff, /*intensity*/ 0.3, /*distance*/ 40, /*decay*/ 0.8 );
							light.position.set( -geocenter_x + v*(featureSize.x/uNum), 6, ceilingRun );
							group.add( light );
						}
						geoFloor = new THREE.BoxGeometry( featureSize.x, 10, ceilingRun );
						var lightMaterial = new THREE.MeshPhongMaterial( { ambient: 0xffffff, color: 0xffffff, side: THREE.DoubleSide, emissive: 0xaaaaaa, shininess: 0, transparent: true, opacity: 0.25 } );
						mshStdFloor = new THREE.Mesh( geoFloor, lightMaterial );
						mshStdFloor.position.set( featureSize.x/2-geocenter_x,  6, ceilingRun/2 );
						group.add( mshStdFloor );
				 	}
                    geometry.vertices.push( new THREE.Vector3 (1500, ceiling_distance, -5) );
                    geometry.vertices.push( new THREE.Vector3 (1500, -1500, -5) );
                    geometry.vertices.push( new THREE.Vector3 (-1500, -1500, -5) );
                    geometry.vertices.push( new THREE.Vector3 (-1500, ceiling_distance, -5) );
                    geometry.faces.push( new THREE.Face3( 0, 1, 2 ) ); // front
                    geometry.faces.push( new THREE.Face3( 0, 2, 3 ) );
                    geometry.computeVertexNormals();
                    meshes_built.push(new THREE.Mesh( geometry, new THREE.MeshPhongMaterial( { color: wall_color, side: THREE.DoubleSide, transparent: true, opacity: wall_opacity } ) ));
                    group.add( meshes_built[meshes_built.length-1] );
                    geometry.dispose();
                    //Render a Ceiling
                    var geometry = new THREE.Geometry();
                    geometry.vertices.push( new THREE.Vector3 (1500, ceiling_distance, 1500) );
                    geometry.vertices.push( new THREE.Vector3 (1500, ceiling_distance, 0) );
                    geometry.vertices.push( new THREE.Vector3 (-1500, ceiling_distance, 0) );
                    geometry.vertices.push( new THREE.Vector3 (-1500, ceiling_distance, 1500) );
                    geometry.faces.push( new THREE.Face3( 0, 1, 2 ) ); // front
                    geometry.faces.push( new THREE.Face3( 0, 2, 3 ) );
                    geometry.computeVertexNormals();
                    meshes_built.push(new THREE.Mesh( geometry, new THREE.MeshPhongMaterial( { color: 0xffffff, side: THREE.DoubleSide, transparent: true, opacity: 0.05 } ) ));
                    group.add( meshes_built[meshes_built.length-1] );
                    geometry.dispose();
                  } else if(featureType == 300) { //Velo
                    //Render a Ceiling
                    var geometry = new THREE.Geometry();
                    geometry.vertices.push( new THREE.Vector3 (max_x - geocenter_x, -0.1, max_z - geocenter_z) );
                    geometry.vertices.push( new THREE.Vector3 (max_x - geocenter_x, -0.1, min_z - geocenter_z) );
                    geometry.vertices.push( new THREE.Vector3 (min_x - geocenter_x, -0.1, min_z - geocenter_z) );
                    geometry.vertices.push( new THREE.Vector3 (min_x - geocenter_x, -0.1, max_z - geocenter_z) );
                    geometry.faces.push( new THREE.Face3( 0, 1, 2 ) ); // front
                    geometry.faces.push( new THREE.Face3( 0, 2, 3 ) );
                    geometry.computeVertexNormals();
                    meshes_built.push(new THREE.Mesh( geometry, new THREE.MeshPhongMaterial( { color: wall_color, side: THREE.DoubleSide, transparent: true, opacity: wall_opacity } ) ));
                    group.add( meshes_built[meshes_built.length-1] );
                    geometry.dispose();
                  } else if(featureType == 400) { //HushBlock
                    //Render a Wall
                    var geometry = new THREE.Geometry();
                    geometry.vertices.push( new THREE.Vector3 (max_x - geocenter_x, max_z - geocenter_y, 1.0) );
                    geometry.vertices.push( new THREE.Vector3 (max_x - geocenter_x, min_z - geocenter_y, 1.0) );
                    geometry.vertices.push( new THREE.Vector3 (min_x - geocenter_x, min_z - geocenter_y, 1.0) );
                    geometry.vertices.push( new THREE.Vector3 (min_x - geocenter_x, max_z - geocenter_y, 1.0) );
                    geometry.faces.push( new THREE.Face3( 0, 1, 2 ) ); // front
                    geometry.faces.push( new THREE.Face3( 0, 2, 3 ) );
                    geometry.computeVertexNormals();
                    meshes_built.push(new THREE.Mesh( geometry, new THREE.MeshPhongMaterial( { color: 0xf1f2f2, side: THREE.DoubleSide, transparent: true, opacity: 0.5 } ) ));
                    group.add( meshes_built[meshes_built.length-1] );
                    geometry.dispose();
                  } else if(featureType == 500) { //Swoon
                   //Render a Ceiling
                    var geometry = new THREE.Geometry();
                    geometry.vertices.push( new THREE.Vector3 (max_x + 6 - geocenter_x, -0.01, max_z + 6 - geocenter_z) );
                    geometry.vertices.push( new THREE.Vector3 (max_x + 6 - geocenter_x, -0.01, min_z - 6 - geocenter_z) );
                    geometry.vertices.push( new THREE.Vector3 (min_x - 6 - geocenter_x, -0.01, min_z - 6 - geocenter_z) );
                    geometry.vertices.push( new THREE.Vector3 (min_x - 6 - geocenter_x, -0.01, max_z + 6 - geocenter_z) );
                    geometry.faces.push( new THREE.Face3( 0, 1, 2 ) ); // front
                    geometry.faces.push( new THREE.Face3( 0, 2, 3 ) );
                    geometry.computeVertexNormals();
                    meshes_built.push(new THREE.Mesh( geometry, new THREE.MeshPhongMaterial( { color: 0xf1f2f2, side: THREE.DoubleSide, transparent: true, opacity: 0.5 } ) ));
                    group.add( meshes_built[meshes_built.length-1] );
                    geometry.dispose();
                  } else if(featureType > 99) { //Clario or Tetria
                    //Render a Ceiling
                    var geometry = new THREE.Geometry();
                    geometry.vertices.push( new THREE.Vector3 (uNum*24/2, -0.1, vNum*24/2) );
                    geometry.vertices.push( new THREE.Vector3 (uNum*24/2, -0.1, -vNum*24/2) );
                    geometry.vertices.push( new THREE.Vector3 (-uNum*24/2, -0.1, -vNum*24/2) );
                    geometry.vertices.push( new THREE.Vector3 (-uNum*24/2, -0.1, vNum*24/2) );
                    geometry.faces.push( new THREE.Face3( 0, 1, 2 ) ); // front
                    geometry.faces.push( new THREE.Face3( 0, 2, 3 ) );
                    geometry.computeVertexNormals();
                    meshes_built.push(new THREE.Mesh( geometry, new THREE.MeshPhongMaterial( { color: 0xf1f2f2, side: THREE.DoubleSide, transparent: true, opacity: 1.0 } ) ));
                    group.add( meshes_built[meshes_built.length-1] );
                    geometry.dispose();
                  } else if(featureType < 0) { //BDC: currently not used but useful for rooms
                    //Render a Wall Plane
                    var geometry = new THREE.Geometry();
                    geometry.vertices.push( new THREE.Vector3 (uNum*24/2, 0, geocenter_z) );
                    geometry.vertices.push( new THREE.Vector3 (uNum*24/2, height, geocenter_z) );
                    geometry.vertices.push( new THREE.Vector3 (-uNum*24/2, height, geocenter_z) );
                    geometry.vertices.push( new THREE.Vector3 (-uNum*24/2, 0, geocenter_z) );
                    geometry.faces.push( new THREE.Face3( 0, 1, 2 ) ); // front
                    geometry.faces.push( new THREE.Face3( 0, 2, 3 ) );
                    geometry.computeVertexNormals();
                    meshes_built.push(new THREE.Mesh( geometry, new THREE.MeshPhongMaterial( { color: 0xd1d3d4, side: THREE.DoubleSide, transparent: true, opacity: 1.0 } ) ));
                    group.add( meshes_built[meshes_built.length-1] );
                    geometry.dispose();
                    //Render a Perpendicular Wall
                    var geometry = new THREE.Geometry();
                    geometry.vertices.push( new THREE.Vector3 (geocenter_x, 0, uNum*24/2) );
                    geometry.vertices.push( new THREE.Vector3 (geocenter_x, height, uNum*24/2) );
                    geometry.vertices.push( new THREE.Vector3 (geocenter_x, height, -uNum*24/2) );
                    geometry.vertices.push( new THREE.Vector3 (geocenter_x, 0, -uNum*24/2) );
                    geometry.faces.push( new THREE.Face3( 0, 1, 2 ) ); // front
                    geometry.faces.push( new THREE.Face3( 0, 2, 3 ) );
                    geometry.computeVertexNormals();
                    meshes_built.push(new THREE.Mesh( geometry, new THREE.MeshPhongMaterial( { color: 0xe6e7e8, side: THREE.DoubleSide, transparent: true, opacity: 1.0 } ) ));
                    group.add( meshes_built[meshes_built.length-1] );
                    geometry.dispose();
                    //Render a Floor
                    var geometry = new THREE.Geometry();
                    geometry.vertices.push( new THREE.Vector3 (uNum*24/2, height, uNum*24/2) );
                    geometry.vertices.push( new THREE.Vector3 (uNum*24/2, height, -uNum*24/2) );
                    geometry.vertices.push( new THREE.Vector3 (-uNum*24/2, height, -uNum*24/2) );
                    geometry.vertices.push( new THREE.Vector3 (-uNum*24/2, height, uNum*24/2) );
                    geometry.faces.push( new THREE.Face3( 0, 1, 2 ) ); // front
                    geometry.faces.push( new THREE.Face3( 0, 2, 3 ) );
                    geometry.computeVertexNormals();
                    meshes_built.push(new THREE.Mesh( geometry, new THREE.MeshPhongMaterial( { color: 0xbbbdc0, side: THREE.DoubleSide, transparent: true, opacity: 1.0 } ) ));
                    group.add( meshes_built[meshes_built.length-1] );
                    geometry.dispose();
                  }

                  if(test == true) {
                    var t1 = performance.now(); //use this for performance timing profiles
                  } else {
                    /*
                     * Very simple Three.js HUD on top of the 3D scene for use as interaction GUI.
                     */
                    // Find distances to upper right corner of canvas
					var x_distance = 3.0*(canvasWidth/canvasHeight);
					var y_distance = 3.0;

                    // Add the reset button.
                    var loader = new THREE.TextureLoader();
                    var texture = loader.load( '/assets/icons/reset.png' );
                    var resetMaterial = new THREE.SpriteMaterial( { map: texture, color: 0xffffff, transparent: true, opacity: 0.50  } );
                    reset_sprite = new THREE.Sprite( resetMaterial );
                    if(featureType == 400) {
                    	reset_sprite.position.set( camera.position.x + (.8*x_distance), camera.position.y - 3, camera.position.z - (.9*y_distance) );
     				} else {
                    	reset_sprite.position.set( camera.position.x + (.8*x_distance), camera.position.y + (.9*y_distance), camera.position.z - 3 );
                    }
					reset_sprite.scale.set( 1.2, .3, 1.0 ); // imageWidth, imageHeight, z
                    objects.push(reset_sprite);
                    scene.add( reset_sprite );
                    resetMaterial.dispose();

                    // Add the 3D button.
                    loader = new THREE.TextureLoader();
                    texture = loader.load( '/assets/icons/3d.png' );
                    resetMaterial = new THREE.SpriteMaterial( { map: texture, color: 0xffffff, transparent: true, opacity: 0.50  } );
                    threed_sprite = new THREE.Sprite( resetMaterial );
                    if(featureType == 400) {
    	                threed_sprite.position.set( camera.position.x + (.84*x_distance), camera.position.y - 3, camera.position.z + (.68*y_distance));                      
                    } else {
	                    threed_sprite.position.set( camera.position.x + (.84*x_distance), camera.position.y - (.68*y_distance), camera.position.z - 3 );                      
                    }
                    threed_sprite.scale.set( .8, .8, 1.0 ); // imageWidth, imageHeight
                    objects.push(threed_sprite);
                    scene.add( threed_sprite );
                    resetMaterial.dispose();

                    // Add the zoom in button.
                    loader = new THREE.TextureLoader();
                    texture = loader.load( '/assets/icons/zoomin.png' );
                    resetMaterial = new THREE.SpriteMaterial( { map: texture, color: 0xffffff, transparent: true, opacity: 0.50  } );
                    zoomin_sprite = new THREE.Sprite( resetMaterial );
                    if(featureType == 400) {
                    	zoomin_sprite.position.set( camera.position.x + (.79*x_distance), camera.position.y - 3, camera.position.z + (.92*y_distance));
                    } else {
                    	zoomin_sprite.position.set( camera.position.x + (.79*x_distance), camera.position.y - (.92*y_distance), camera.position.z - 3 );
                    }
                    zoomin_sprite.scale.set( .3, .3, 1.0 ); // imageWidth, imageHeight
                    objects.push(zoomin_sprite);
                    scene.add( zoomin_sprite );
                    resetMaterial.dispose();

                    // Add the zoom out button.
                    loader = new THREE.TextureLoader();
                    texture = loader.load( '/assets/icons/zoomout.png' );
                    resetMaterial = new THREE.SpriteMaterial( { map: texture, color: 0xffffff, transparent: true, opacity: 0.50  } );
                    zoomout_sprite = new THREE.Sprite( resetMaterial );
                    if(featureType == 400) {
                    	zoomout_sprite.position.set( camera.position.x + (.89*x_distance), camera.position.y - 3, camera.position.z + (.92*y_distance));
                    } else {
                    	zoomout_sprite.position.set( camera.position.x + (.89*x_distance), camera.position.y - (.92*y_distance), camera.position.z - 3 );
                    }
                    zoomout_sprite.scale.set( .3, .3, 1.0 ); // imageWidth, imageHeight
                    objects.push(zoomout_sprite);
                    scene.add( zoomout_sprite );
                    resetMaterial.dispose();
                    /* end HUD code */
                  }

				  renderer = new THREE.WebGLRenderer( { preserveDrawingBuffer: true, antialias: true } );
                  renderer.autoClear = false;
                  renderer.setClearColor( 0xf0f0f0 );
                  renderer.setPixelRatio( window.devicePixelRatio );
                  renderer.setSize( canvasWidth, canvasHeight );

                  //Remove previous renderings  TODO: break out this code into pieces so don't need to duplicate each time
                  while (document.getElementById('visualization').firstChild) {
                      document.getElementById('visualization').removeChild(document.getElementById('visualization').firstChild);
                  }
                  document.getElementById('visualization').appendChild( renderer.domElement );

                  document.getElementById('visualization').addEventListener( 'mousedown', onDocumentMouseDown, false );
                  document.getElementById('visualization').addEventListener( 'touchstart', onDocumentTouchStart, false );
                  document.getElementById('visualization').addEventListener( 'touchmove', onDocumentTouchMove, false );
                  document.getElementById('visualization').addEventListener( 'mousewheel', onMouseWheel, false );
                  document.getElementById('visualization').addEventListener( 'DOMMouseScroll', onMouseWheel, false ); // firefox

                  window.addEventListener( 'resize', onWindowResize, false );
                }
                if(test == true) {
                    console.log('EXECUTION TIME:' + (performance.now() - t0)+ " milliseconds.");
                    return ("syd-vis t0:" + (performance.now() - t0) + " milliseconds.");
                }
            },

            /*
             * Get back all the OpenGL for meshes that were created in the scene
             */
            destroyFeature: function() {
                for(i=0;i<meshes_built.length;i++) {
                    scene.remove(meshes_built[i]);
                }
                scene.remove(group);
            },

            /*
             * Set the material property
             */
            SetMaterial: function(material_string) {
                material = material_string;
            },
            /*
             * Set the feature type: 0-StraightPartition, 1-ArcPartition, 2-Facing, 3-Transition, 4-Ceiling, 5-BentPartition
             */
            SetFeatureType: function(feature_type) {
                featureType = feature_type;
            },

            ResetView: function() {
                resetView();
            },

            ZoomIn: function() {
                zoomIn();
            },

            ZoomOut: function() {
                zoomOut();
            },

            PitchUp: function() {
                pitchUp();
            },

            PitchDown: function() {
                pitchDown();
            },

            YawRight: function() {
                yawRight();
            },

            YawLeft: function() {
                yawLeft();
            },

            /*
                * Take a snapshot of the screen
                */
            TakeSnapshot: function(angle) {
                resetView();
                group.rotation.y = parseInt(angle) * Math.PI/180.0;
                targetRotation = group.rotation.y;

                // Render the Seeyond scene.
                renderer.clear();
                renderer.render( scene, camera );
                imgData = renderer.domElement.toDataURL();
                snapshot = imgData;
                console.log("Snapshot taken:\n------------\n");
                console.log(imgData);
                return snapshot;
            },
            /*
             * Get the last snapshot taken
             */
            GetLastSnapshot: function() {
                console.log(snapshot);
                return snapshot;
            },
            /*
             * Get the feature's width
             */           
            GetBoundingBoxWidth: function() {
                return featureSize.x;
            },
            /*
             * Get the feature's height
             */
            GetBoundingBoxHeight: function() {
                return featureSize.y;
            },
            /*
             * Get the feature's depth
             */
            GetBoundingBoxDepth: function() {
                return featureSize.z;
            },
        }
    }
})(typeof exports === 'undefined'? this['Syd']={}: exports);

/*
 * Generate boxes from front and back surfaces
 */
function GenerateBoxesFromSurfaces(front, back, uNum, vNum) {
    var boxes = new Array();
    for(var i=0;i<uNum;i++) {
        for(var j=0;j<vNum;j++) {
            box = new Array();
            box.push(front[i][j]);
            box.push(front[i+1][j]);
            box.push(front[i+1][j+1]);
            box.push(front[i][j+1]);
            box.push(back[i][j]);
            box.push(back[i+1][j]);
            box.push(back[i+1][j+1]);
            box.push(back[i][j+1]);
            boxes.push(box);
        }
    }
    return boxes;
};

function GenerateMidSeamFromBoxes(geometry, v1, v2, v3, v4, b1, seg_type, bAmt) {
	var vec = new THREE.Vector3 (geometry.vertices[v1].x-geometry.vertices[v3].x, geometry.vertices[v1].y-geometry.vertices[v3].y, geometry.vertices[v1].z-geometry.vertices[v3].z);
	vec.setLength(bevelAmt*.707);
	var bck_f = new THREE.Vector3 ((geometry.vertices[v1].x+geometry.vertices[v3].x)/2 - vec.x, (geometry.vertices[v1].y+geometry.vertices[v3].y)/2 - vec.y, (geometry.vertices[v1].z+geometry.vertices[v3].z)/2 - vec.z);
	var front_a = new THREE.Vector3 ((geometry.vertices[v1].x+geometry.vertices[v3].x)/2 + vec.x, (geometry.vertices[v1].y+geometry.vertices[v3].y)/2 + vec.y, (geometry.vertices[v1].z+geometry.vertices[v3].z)/2 + vec.z);
	geometry.vertices.push(front_a);
	geometry.vertices.push(bck_f);							
	vec = new THREE.Vector3 (geometry.vertices[v2].x-geometry.vertices[v4].x, geometry.vertices[v2].y-geometry.vertices[v4].y, geometry.vertices[v2].z-geometry.vertices[v4].z);
	vec.setLength(bevelAmt*.707);
	bck_f = new THREE.Vector3 ((geometry.vertices[v2].x+geometry.vertices[v4].x)/2 - vec.x, (geometry.vertices[v2].y+geometry.vertices[v4].y)/2 - vec.y, (geometry.vertices[v2].z+geometry.vertices[v4].z)/2 - vec.z);
	var front_b = new THREE.Vector3 ((geometry.vertices[v2].x+geometry.vertices[v4].x)/2 + vec.x, (geometry.vertices[v2].y+geometry.vertices[v4].y)/2 + vec.y, (geometry.vertices[v2].z+geometry.vertices[v4].z)/2 + vec.z);
	geometry.vertices.push(front_b);
	geometry.vertices.push(bck_f);
	//center line indent vertices:
	vec = new THREE.Vector3 (geometry.vertices[v1].x-geometry.vertices[b1].x, geometry.vertices[v1].y-geometry.vertices[b1].y, geometry.vertices[v1].z-geometry.vertices[b1].z);
	vec.setLength(bevelAmt*.707);
	if(seg_type=="ltc" && seg_type=="rtc") {
		bck_f = new THREE.Vector3 ((geometry.vertices[v1].x+geometry.vertices[v3].x)/2 - vec.x, (geometry.vertices[v1].y+geometry.vertices[v3].y+bAmt*3)/2 - vec.y, (geometry.vertices[v1].z+geometry.vertices[v3].z)/2 - vec.z);
	} else {
		bck_f = new THREE.Vector3 ((geometry.vertices[v1].x+geometry.vertices[v3].x)/2 - vec.x, (geometry.vertices[v1].y+geometry.vertices[v3].y)/2 - vec.y, (geometry.vertices[v1].z+geometry.vertices[v3].z)/2 - vec.z);
	}
	geometry.vertices.push(bck_f);
	if(seg_type=="lbc" && seg_type=="rbc") {
		bck_f = new THREE.Vector3 ((geometry.vertices[v2].x+geometry.vertices[v4].x)/2 - vec.x, (geometry.vertices[v2].y+geometry.vertices[v4].y-bAmt*3)/2 - vec.y, (geometry.vertices[v2].z+geometry.vertices[v4].z)/2 - vec.z);
	} else {
		bck_f = new THREE.Vector3 ((geometry.vertices[v2].x+geometry.vertices[v4].x)/2 - vec.x, (geometry.vertices[v2].y+geometry.vertices[v4].y)/2 - vec.y, (geometry.vertices[v2].z+geometry.vertices[v4].z)/2 - vec.z);
	}
	geometry.vertices.push(bck_f);	
};

/* 
 * Generate a Clario Tile
 */
function GenerateClarioTile(geometry, depth, x, y, w, h, rotation) {
    var indx = 0;
    var slant = 1.5;
    var straightsleft = [[3,0,15,24],[9,0,21,24]];
    var straightsright = [[15,0,3,24],[21,0,9,24]];
    var bentsleft = [[9,24,0,6,3,0],[3,24,0,18,9,0]];
    var bentsright = [[15,24,24,6,21,0],[21,24,24,18,15,0]];
    var bentsright48 = [[39,0,48,18,45,24],[39,24,48,6,45,0]];
    if((w==24 && (rotation == 90 || rotation == 270)) || (w==48)) {
		straightsleft = [[0,3,24,15],[0,9,24,21]];
		straightsright = [[0,15,24,3],[0,21,24,9]];
		bentsleft = [[24,9,6,0,0,3],[24,3,18,0,0,9]];
		bentstop48 = [[48,9,30,0,24,3],[48,3,42,0,36,3]];
		bentsbottomleft48 = [[0,15,18,24,24,21],[0,21,6,24,12,21]];
		bentsbottomright48 = [[24,21,30,24,48,15],[36,21,42,24,48,21]];
		bentsright = [[24,15,6,24,0,21],[24,21,18,24,0,15]];  	
    }
    //Render the straight components
	for(i=0; i<2; i++) {
		geometry.vertices.push( new THREE.Vector3 (x*24 + straightsleft[i][0] - geocenter_x + slant/2, 0, y*24 + straightsleft[i][1] - geocenter_z) );
		geometry.vertices.push( new THREE.Vector3 (x*24 + straightsleft[i][0] - geocenter_x - slant/2, depth, y*24 + straightsleft[i][1] - geocenter_z) );
		geometry.vertices.push( new THREE.Vector3 (x*24 + straightsleft[i][2] - geocenter_x - slant/2, depth, y*24 + straightsleft[i][3] - geocenter_z) );
		geometry.vertices.push( new THREE.Vector3 (x*24 + straightsleft[i][2] - geocenter_x + slant/2, 0, y*24 + straightsleft[i][3] - geocenter_z) );
		if(w==48) {
			geometry.vertices.push( new THREE.Vector3 (x*24+12 + straightsleft[i][0] - geocenter_x + slant/2, 0, y*24 + straightsleft[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24+12 + straightsleft[i][0] - geocenter_x - slant/2, depth, y*24 + straightsleft[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24+12 + straightsleft[i][2] - geocenter_x - slant/2, depth, y*24 + straightsleft[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24+12 + straightsleft[i][2] - geocenter_x + slant/2, 0, y*24 + straightsleft[i][3] - geocenter_z) );	
			geometry.vertices.push( new THREE.Vector3 (x*24+24 + straightsleft[i][0] - geocenter_x + slant/2, 0, y*24 + straightsleft[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24+24 + straightsleft[i][0] - geocenter_x - slant/2, depth, y*24 + straightsleft[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24+24 + straightsleft[i][2] - geocenter_x - slant/2, depth, y*24 + straightsleft[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24+24 + straightsleft[i][2] - geocenter_x + slant/2, 0, y*24 + straightsleft[i][3] - geocenter_z) );	
		}
	}
	for(i=0; i<2; i++) {
		geometry.vertices.push( new THREE.Vector3 (x*24 + straightsright[i][0] - geocenter_x - slant/2, 0, y*24 + straightsright[i][1] - geocenter_z) );
		geometry.vertices.push( new THREE.Vector3 (x*24 + straightsright[i][0] - geocenter_x + slant/2, depth, y*24 + straightsright[i][1] - geocenter_z) );
		geometry.vertices.push( new THREE.Vector3 (x*24 + straightsright[i][2] - geocenter_x + slant/2, depth, y*24 + straightsright[i][3] - geocenter_z) );
		geometry.vertices.push( new THREE.Vector3 (x*24 + straightsright[i][2] - geocenter_x - slant/2, 0, y*24 + straightsright[i][3] - geocenter_z) );
		if(w==48) {
			geometry.vertices.push( new THREE.Vector3 (x*24+12 + straightsright[i][0] - geocenter_x - slant/2, 0, y*24 + straightsright[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24+12 + straightsright[i][0] - geocenter_x + slant/2, depth, y*24 + straightsright[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24+12 + straightsright[i][2] - geocenter_x + slant/2, depth, y*24 + straightsright[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24+12 + straightsright[i][2] - geocenter_x - slant/2, 0, y*24 + straightsright[i][3] - geocenter_z) );	
			geometry.vertices.push( new THREE.Vector3 (x*24+24 + straightsright[i][0] - geocenter_x - slant/2, 0, y*24 + straightsright[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24+24 + straightsright[i][0] - geocenter_x + slant/2, depth, y*24 + straightsright[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24+24 + straightsright[i][2] - geocenter_x + slant/2, depth, y*24 + straightsright[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24+24 + straightsright[i][2] - geocenter_x - slant/2, 0, y*24 + straightsright[i][3] - geocenter_z) );	
		}
	}
	//Render the bent components
	for(i=0; i<2; i++) {
		geometry.vertices.push( new THREE.Vector3 (x*24 + bentsleft[i][0] - geocenter_x + slant/2, 0, y*24 + bentsleft[i][1] - geocenter_z) );
		geometry.vertices.push( new THREE.Vector3 (x*24 + bentsleft[i][0] - geocenter_x - slant/2, depth, y*24 + bentsleft[i][1] - geocenter_z) );
		geometry.vertices.push( new THREE.Vector3 (x*24 + bentsleft[i][2] - geocenter_x, depth, y*24 + bentsleft[i][3] - geocenter_z) );
		geometry.vertices.push( new THREE.Vector3 (x*24 + bentsleft[i][2] - geocenter_x, 0, y*24 + bentsleft[i][3] - geocenter_z) );
		//Non-parallelogram geometry
		geometry.vertices.push( new THREE.Vector3 (x*24 + bentsleft[i][2] - geocenter_x, 0, y*24 + bentsleft[i][3] - geocenter_z) );
		geometry.vertices.push( new THREE.Vector3 (x*24 + bentsleft[i][2] - geocenter_x, depth, y*24 + bentsleft[i][3] - geocenter_z) );
		geometry.vertices.push( new THREE.Vector3 (x*24 + bentsleft[i][4] - geocenter_x + slant/2, depth, y*24 + bentsleft[i][5] - geocenter_z) );
		geometry.vertices.push( new THREE.Vector3 (x*24 + bentsleft[i][4] - geocenter_x - slant/2, 0, y*24 + bentsleft[i][5] - geocenter_z) );
		if(w==24) {
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsright[i][0] - geocenter_x - slant/2, 0, y*24 + bentsright[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsright[i][0] - geocenter_x + slant/2, depth, y*24 + bentsright[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsright[i][2] - geocenter_x, depth, y*24 + bentsright[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsright[i][2] - geocenter_x, 0, y*24 + bentsright[i][3] - geocenter_z) );
			//Non-parallelogram geometry
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsright[i][2] - geocenter_x, 0, y*24 + bentsright[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsright[i][2] - geocenter_x, depth, y*24 + bentsright[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsright[i][4] - geocenter_x - slant/2, depth, y*24 + bentsright[i][5] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsright[i][4] - geocenter_x + slant/2, 0, y*24 + bentsright[i][5] - geocenter_z) );
		} else {
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentstop48[i][0] - geocenter_x + slant/2, 0, y*24 + bentstop48[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentstop48[i][0] - geocenter_x - slant/2, depth, y*24 + bentstop48[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentstop48[i][2] - geocenter_x, depth, y*24 + bentstop48[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentstop48[i][2] - geocenter_x, 0, y*24 + bentstop48[i][3] - geocenter_z) );

			geometry.vertices.push( new THREE.Vector3 (x*24 + bentstop48[i][2] - geocenter_x, 0, y*24 + bentstop48[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentstop48[i][2] - geocenter_x, depth, y*24 + bentstop48[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentstop48[i][4] - geocenter_x + slant/2, depth, y*24 + bentstop48[i][5] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentstop48[i][4] - geocenter_x - slant/2, 0, y*24 + bentstop48[i][5] - geocenter_z) );

			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsbottomleft48[i][0] - geocenter_x - slant/2, 0, y*24 + bentsbottomleft48[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsbottomleft48[i][0] - geocenter_x + slant/2, depth, y*24 + bentsbottomleft48[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsbottomleft48[i][2] - geocenter_x, depth, y*24 + bentsbottomleft48[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsbottomleft48[i][2] - geocenter_x, 0, y*24 + bentsbottomleft48[i][3] - geocenter_z) );

			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsbottomleft48[i][2] - geocenter_x, 0, y*24 + bentsbottomleft48[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsbottomleft48[i][2] - geocenter_x, depth, y*24 + bentsbottomleft48[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsbottomleft48[i][4] - geocenter_x + slant/2, depth, y*24 + bentsbottomleft48[i][5] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsbottomleft48[i][4] - geocenter_x - slant/2, 0, y*24 + bentsbottomleft48[i][5] - geocenter_z) );

			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsbottomright48[i][0] - geocenter_x + slant/2, 0, y*24 + bentsbottomright48[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsbottomright48[i][0] - geocenter_x - slant/2, depth, y*24 + bentsbottomright48[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsbottomright48[i][2] - geocenter_x, depth, y*24 + bentsbottomright48[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsbottomright48[i][2] - geocenter_x, 0, y*24 + bentsbottomright48[i][3] - geocenter_z) );

			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsbottomright48[i][2] - geocenter_x, 0, y*24 + bentsbottomright48[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsbottomright48[i][2] - geocenter_x, depth, y*24 + bentsbottomright48[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsbottomright48[i][4] - geocenter_x - slant/2, depth, y*24 + bentsbottomright48[i][5] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsbottomright48[i][4] - geocenter_x + slant/2, 0, y*24 + bentsbottomright48[i][5] - geocenter_z) );
		}
	}
	if (h==48) {
    //Render the straight components
		for(i=0; i<2; i++) {
			geometry.vertices.push( new THREE.Vector3 (x*24 + straightsleft[i][0] - geocenter_x + slant/2, 0, (y+1)*24 + straightsleft[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + straightsleft[i][0] - geocenter_x - slant/2, depth, (y+1)*24 + straightsleft[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + straightsleft[i][2] - geocenter_x - slant/2, depth, (y+1)*24 + straightsleft[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + straightsleft[i][2] - geocenter_x + slant/2, 0, (y+1)*24 + straightsleft[i][3] - geocenter_z) );
		}
		for(i=0; i<2; i++) {
			geometry.vertices.push( new THREE.Vector3 (x*24 + straightsright[i][0] - geocenter_x + slant/2, 0, (y+1)*24 + straightsright[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + straightsright[i][0] - geocenter_x - slant/2, depth, (y+1)*24 + straightsright[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + straightsright[i][2] - geocenter_x - slant/2, depth, (y+1)*24 + straightsright[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + straightsright[i][2] - geocenter_x + slant/2, 0, (y+1)*24 + straightsright[i][3] - geocenter_z) );
		}
		//Render the bent components
		for(i=0; i<2; i++) {
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsleft[i][0] - geocenter_x + slant/2, 0, (y+1)*24 + bentsleft[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsleft[i][0] - geocenter_x - slant/2, depth, (y+1)*24 + bentsleft[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsleft[i][2] - geocenter_x - slant/2, depth, (y+1)*24 + bentsleft[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsleft[i][2] - geocenter_x + slant/2, 0, (y+1)*24 + bentsleft[i][3] - geocenter_z) );

			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsleft[i][2] - geocenter_x + slant/2, 0, (y+1)*24 + bentsleft[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsleft[i][2] - geocenter_x - slant/2, depth, (y+1)*24 + bentsleft[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsleft[i][4] - geocenter_x - slant/2, depth, (y+1)*24 + bentsleft[i][5] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsleft[i][4] - geocenter_x + slant/2, 0, (y+1)*24 + bentsleft[i][5] - geocenter_z) );

			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsright[i][0] - geocenter_x + slant/2, 0, (y+1)*24 + bentsright[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsright[i][0] - geocenter_x - slant/2, depth, (y+1)*24 + bentsright[i][1] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsright[i][2] - geocenter_x - slant/2, depth, (y+1)*24 + bentsright[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsright[i][2] - geocenter_x + slant/2, 0, (y+1)*24 + bentsright[i][3] - geocenter_z) );

			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsright[i][2] - geocenter_x + slant/2, 0, (y+1)*24 + bentsright[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsright[i][2] - geocenter_x - slant/2, depth, (y+1)*24 + bentsright[i][3] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsright[i][4] - geocenter_x - slant/2, depth, (y+1)*24 + bentsright[i][5] - geocenter_z) );
			geometry.vertices.push( new THREE.Vector3 (x*24 + bentsright[i][4] - geocenter_x + slant/2, 0, (y+1)*24 + bentsright[i][5] - geocenter_z) );
		}
	}
	for(i=0; i<12; i++) {
		geometry.faces.push( new THREE.Face3( indx + 0, indx + 1, indx + 2 ) ); // left slat
		geometry.faces.push( new THREE.Face3( indx + 0, indx + 2, indx + 3 ) );
		indx += 4;
	}
	if (w==48) {
		for(i=0; i<16; i++) {
			geometry.faces.push( new THREE.Face3( indx + 0, indx + 1, indx + 2 ) ); // left slat
			geometry.faces.push( new THREE.Face3( indx + 0, indx + 2, indx + 3 ) );
			indx += 4;
		}		
	}
	if (h==48) {
		for(i=0; i<12; i++) {
			geometry.faces.push( new THREE.Face3( indx + 0, indx + 1, indx + 2 ) ); // left slat
			geometry.faces.push( new THREE.Face3( indx + 0, indx + 2, indx + 3 ) );
			indx += 4;
		}		
	}
	return geometry;
}

/* 
 * Generate a Velo Tile
 */
function GenerateVeloTile(geometry, feature, boxNum, depth) {
	
	var cylinder = [new THREE.Geometry(), new THREE.Geometry(), new THREE.Geometry(), new THREE.Geometry(), new THREE.Geometry()];
    var cylindermaterial = new THREE.LineBasicMaterial({color: 0x000000, linewidth: 1});
    var hang_dist = 12; //the distance to hang from the ceiling

	var row = feature.getBox(boxNum).getFloats()[15];
	var column = feature.getBox(boxNum).getFloats()[17];
    var rotation = feature.getBox(boxNum).getFloats()[16];
	var center_x = (feature.getBox(boxNum).getFloats()[9] + feature.getBox(boxNum).getFloats()[12])/2.0;
	var center_y = 0.0;
	var center_z = (feature.getBox(boxNum).getFloats()[2] + feature.getBox(boxNum).getFloats()[8])/2.0;
	
	var upperleft_x = column * (47.82);
	var upperleft_z = row * (23.92);
	if(row % 2 == 0) {
		upperleft_x -= 23.92;
	}
	if ((rotation < 4.8 && rotation > 4.5) || (rotation < -1.4 && rotation > -1.7) || (rotation==270) || (rotation==-90)) {  //270
		coords=[[0-11.94, 3.985-11.94],[0-11.94, 11.96-11.94],[0-11.94, 19.935-11.94],[11.94-11.94, 23.92-11.94],[15.94-11.94, 11.96-11.94],[11.96-11.94, 0-11.94],[7.97-11.94, 11.96-11.94]];
	} else if ((rotation > 3.0 && rotation < 3.3) || (rotation > -3.3 && rotation < -3.0) || (rotation==180) || (rotation==-180)) {  //180
		coords=[[3.985, 15.94-15.94],[11.96, 15.94-15.94],[19.935, 15.94-15.94],[23.92, 4-15.94],[11.96, 0-15.94],[0, 4-15.94],[11.96, 7.97-15.94]];
	} else if ((rotation > 1.4 && rotation < 1.7) || (rotation < -4.5 && rotation > -4.8) || (rotation==90) || (rotation==-270)) { //90
		coords = [[3.985+19.94, 0-11.94],[0+19.94, 11.96-11.94],[3.985+19.94, 23.92-11.94],[19.94+15.94, 19.935-11.94],[19.94+15.94, 11.96-11.94],[19.94+15.94, 3.985-11.94],[7.97+19.94, 11.96-11.94]];
	} else {  //0
		coords=[[3.985, 0],[11.96, 0],[19.935, 0],[23.92, 12],[11.96, 16],[0, 12],[11.96, 7.97]];
	}

	for (i=0; i<6; i++) {
		if(coords[i][0] + upperleft_x > max_x) {
			max_x = coords[i][0] + upperleft_x;
		}
		if(coords[i][0] + upperleft_x < min_x) {
			min_x = coords[i][0] + upperleft_x;
		}
		if(coords[i][1] + upperleft_z  > max_z) {
			max_z = coords[i][1] + upperleft_z;
		}		
		if(coords[i][1] + upperleft_z  < min_z) {
			min_z = coords[i][1] + upperleft_z;
		}
	}

	geometry.vertices.push( new THREE.Vector3 (coords[0][0] + upperleft_x - geocenter_x,  hang_dist, coords[0][1] + upperleft_z - geocenter_z) );
	geometry.vertices.push( new THREE.Vector3 (coords[1][0] + upperleft_x - geocenter_x,  hang_dist, coords[1][1] + upperleft_z - geocenter_z) );
	geometry.vertices.push( new THREE.Vector3 (coords[2][0] + upperleft_x - geocenter_x,  hang_dist, coords[2][1] + upperleft_z - geocenter_z) );
	geometry.vertices.push( new THREE.Vector3 (coords[3][0] + upperleft_x - geocenter_x,  hang_dist, coords[3][1] + upperleft_z - geocenter_z) );
	geometry.vertices.push( new THREE.Vector3 (coords[4][0] + upperleft_x - geocenter_x,  hang_dist, coords[4][1] + upperleft_z - geocenter_z) );
	geometry.vertices.push( new THREE.Vector3 (coords[5][0] + upperleft_x - geocenter_x,  hang_dist, coords[5][1] + upperleft_z - geocenter_z) );
	geometry.vertices.push( new THREE.Vector3 (coords[6][0] + upperleft_x - geocenter_x, depth + hang_dist, coords[6][1] + upperleft_z - geocenter_z) );

	geometry.faces.push( new THREE.Face3( 0, 1, 6 ) );
	geometry.faces.push( new THREE.Face3( 1, 2, 6 ) );
	geometry.faces.push( new THREE.Face3( 2, 3, 6 ) );
	geometry.faces.push( new THREE.Face3( 3, 4, 6 ) );
	geometry.faces.push( new THREE.Face3( 4, 5, 6 ) );
	geometry.faces.push( new THREE.Face3( 5, 0, 6 ) );

	for (i=0; i<6; i++) {
		if(i==5) {
			geometry.faceVertexUvs[0].push([	
				new THREE.Vector2(coords[i][0] + upperleft_x - geocenter_x,coords[0][1] + upperleft_z - geocenter_z),
				new THREE.Vector2(coords[i][0] + upperleft_x - geocenter_x,coords[0][1] + upperleft_z - geocenter_z),
				new THREE.Vector2(coords[6][0] + upperleft_x - geocenter_x,coords[6][1] + upperleft_z - geocenter_z)
			]);
		} else {
			geometry.faceVertexUvs[0].push([
				new THREE.Vector2(coords[i][0] + upperleft_x - geocenter_x,coords[i+1][1] + upperleft_z - geocenter_z),
				new THREE.Vector2(coords[i][0] + upperleft_x - geocenter_x,coords[i+1][1] + upperleft_z - geocenter_z),
				new THREE.Vector2(coords[6][0] + upperleft_x - geocenter_x,coords[6][1] + upperleft_z - geocenter_z)
			]);
		}
	}
	
	cylinder[0].vertices.push(new THREE.Vector3(coords[1][0] + upperleft_x - geocenter_x, 0.0, coords[1][1] + upperleft_z - geocenter_z));
	cylinder[0].vertices.push(new THREE.Vector3(coords[1][0] + upperleft_x - geocenter_x, 11.8, coords[1][1] + upperleft_z - geocenter_z));
	group.add(new THREE.Line(cylinder[0], new THREE.LineBasicMaterial({color: 0x000000, linewidth: 1})));
	cylinder[1].vertices.push(new THREE.Vector3(((coords[2][0] + upperleft_x - geocenter_x) + (coords[3][0] + upperleft_x - geocenter_x))/2, 0.0, ((coords[2][1] + upperleft_z - geocenter_z) + (coords[3][1] + upperleft_z - geocenter_z))/2));
	cylinder[1].vertices.push(new THREE.Vector3(((coords[2][0] + upperleft_x - geocenter_x) + (coords[3][0] + upperleft_x - geocenter_x))/2, 11.8, ((coords[2][1] + upperleft_z - geocenter_z) + (coords[3][1] + upperleft_z - geocenter_z))/2));
	group.add(new THREE.Line(cylinder[1], new THREE.LineBasicMaterial({color: 0x000000, linewidth: 1})));
	cylinder[2].vertices.push(new THREE.Vector3(((coords[3][0] + upperleft_x - geocenter_x) + (coords[4][0] + upperleft_x - geocenter_x))/2, 0.0, ((coords[3][1] + upperleft_z - geocenter_z) + (coords[4][1] + upperleft_z - geocenter_z))/2));
	cylinder[2].vertices.push(new THREE.Vector3(((coords[3][0] + upperleft_x - geocenter_x) + (coords[4][0] + upperleft_x - geocenter_x))/2, 11.8, ((coords[3][1] + upperleft_z - geocenter_z) + (coords[4][1] + upperleft_z - geocenter_z))/2));
	group.add(new THREE.Line(cylinder[2], new THREE.LineBasicMaterial({color: 0x000000, linewidth: 1})));
	cylinder[3].vertices.push(new THREE.Vector3(((coords[4][0] + upperleft_x - geocenter_x) + (coords[5][0] + upperleft_x - geocenter_x))/2, 0.0, ((coords[4][1] + upperleft_z - geocenter_z) + (coords[5][1] + upperleft_z - geocenter_z))/2));
	cylinder[3].vertices.push(new THREE.Vector3(((coords[4][0] + upperleft_x - geocenter_x) + (coords[5][0] + upperleft_x - geocenter_x))/2, 11.8, ((coords[4][1] + upperleft_z - geocenter_z) + (coords[5][1] + upperleft_z - geocenter_z))/2));
	group.add(new THREE.Line(cylinder[3], new THREE.LineBasicMaterial({color: 0x000000, linewidth: 1})));
	cylinder[4].vertices.push(new THREE.Vector3(((coords[5][0] + upperleft_x - geocenter_x) + (coords[0][0] + upperleft_x - geocenter_x))/2, 0.0, ((coords[5][1] + upperleft_z - geocenter_z) + (coords[0][1] + upperleft_z - geocenter_z))/2));
	cylinder[4].vertices.push(new THREE.Vector3(((coords[5][0] + upperleft_x - geocenter_x) + (coords[0][0] + upperleft_x - geocenter_x))/2, 11.8, ((coords[5][1] + upperleft_z - geocenter_z) + (coords[0][1] + upperleft_z - geocenter_z))/2));
	group.add(new THREE.Line(cylinder[4], new THREE.LineBasicMaterial({color: 0x000000, linewidth: 1})));

	for (i=0; i<5; i++) {
		cylinder[i].dispose();
	}
	cylindermaterial.dispose();
	
	return geometry;
}

/* 
 * Generate a HushBlock Tile
 */
function GenerateHushBlockTile(geometry, feature, boxNum, depth) {
	
	var row = feature.getBox(boxNum).getFloats()[15];
	var column = feature.getBox(boxNum).getFloats()[17];
    var rotation = feature.getBox(boxNum).getFloats()[16];
	var center_x = (feature.getBox(boxNum).getFloats()[9] + feature.getBox(boxNum).getFloats()[12])/2.0;
	var center_y = 0.0;
	var center_z = (feature.getBox(boxNum).getFloats()[2] + feature.getBox(boxNum).getFloats()[8])/2.0;
	var b_amt = 0.5;

	coords = [feature.getBox(boxNum).getFloats()[0], feature.getBox(boxNum).getFloats()[2],
			  feature.getBox(boxNum).getFloats()[3], feature.getBox(boxNum).getFloats()[5],
			  feature.getBox(boxNum).getFloats()[6], feature.getBox(boxNum).getFloats()[8],
			  feature.getBox(boxNum).getFloats()[9], feature.getBox(boxNum).getFloats()[11]];

	if(coords[4] > max_x) {
		max_x = coords[4];
	}
	if(coords[5] > max_z) {
		max_z = coords[5];
	}	
	if(coords[0] < min_x) {
		min_x = coords[0];
	}	
	if(coords[1] < min_z) {
		min_z = coords[1];
	}
	
	geometry.vertices.push( new THREE.Vector3 (coords[0] - geocenter_x + b_amt, coords[1] - geocenter_y + b_amt, 0.0) );
	geometry.vertices.push( new THREE.Vector3 (coords[2] - geocenter_x + b_amt, coords[3] - geocenter_y - b_amt, 0.0) );
	geometry.vertices.push( new THREE.Vector3 (coords[4] - geocenter_x - b_amt, coords[5] - geocenter_y - b_amt, 0.0) );
	geometry.vertices.push( new THREE.Vector3 (coords[6] - geocenter_x - b_amt, coords[7] - geocenter_y + b_amt, 0.0) );
	geometry.vertices.push( new THREE.Vector3 (coords[0] - geocenter_x + b_amt, coords[1] - geocenter_y + b_amt, feature.getBox(boxNum).getFloats()[1]) );
	geometry.vertices.push( new THREE.Vector3 (coords[2] - geocenter_x + b_amt, coords[3] - geocenter_y - b_amt, feature.getBox(boxNum).getFloats()[4]) );
	geometry.vertices.push( new THREE.Vector3 (coords[4] - geocenter_x - b_amt, coords[5] - geocenter_y - b_amt, feature.getBox(boxNum).getFloats()[7]) );
	geometry.vertices.push( new THREE.Vector3 (coords[6] - geocenter_x - b_amt, coords[7] - geocenter_y + b_amt, feature.getBox(boxNum).getFloats()[10]) );

	geometry.faces.push( new THREE.Face3( 0, 1, 2) ); //back
	geometry.faces.push( new THREE.Face3( 0, 2, 3) );
	geometry.faces.push( new THREE.Face3( 4, 5, 6) ); //front
	geometry.faces.push( new THREE.Face3( 4, 6, 7) );	
	geometry.faces.push( new THREE.Face3( 0, 3, 4) ); //left
	geometry.faces.push( new THREE.Face3( 3, 7, 4) );
	geometry.faces.push( new THREE.Face3( 1, 2, 5) ); //right
	geometry.faces.push( new THREE.Face3( 2, 5, 6) );
	geometry.faces.push( new THREE.Face3( 2, 3, 6) ); //bottom
	geometry.faces.push( new THREE.Face3( 3, 7, 6) );
	geometry.faces.push( new THREE.Face3( 1, 0, 5) ); //top
	geometry.faces.push( new THREE.Face3( 0, 5, 4) );
	
	assignUVs(geometry);
	return geometry;
}

/* 
 * Generate a Swoon Tile
 */
function GenerateSwoonTile(geometry, feature, boxNum, depth) {
	
    var hang_dist = 0.5; //the distance to hang from wall

	var column = feature.getBox(boxNum).getFloats()[15];
	var row = feature.getBox(boxNum).getFloats()[17];
    var rotation = feature.getBox(boxNum).getFloats()[16];
	var coords = new Array();
	var bev = .6;
	var bmi = .27;
	var bmj = .19;
	var rot_adj_base = [[0,0],[0,0],[0,0],[0,0]];
	var rot_adj = [[0,0-bev],[0-bev,0],[0,+bev],[0+bev,0]];
		
	var upper_x = (column) * (8.66);
	var upper_y = (row) * (5.21+2.605);

	if(row % 2 == 1) {	
		upper_x += 4.33
	}
	if(rotation > 50 && rotation < 70) {
		rot_adj_base = [[0,5.257],[-8.65,5.257],[-4.33,2.605],[4.337,2.605]];
		rot_adj = [[0-bmi,5.257-bev],[-8.65+bmi,5.138-bmi],[-4.33+bmi,2.605+bev],[4.337-bmi,2.605+bmi]];

	} else if(rotation < -50 && rotation > -70) {
		rot_adj_base = [[0,5.257],[0,5.257],[4.33,2.605],[4.33,2.605]];
		rot_adj = [[0+bmi,5.257-bev],[0-bmj,5.23-bmi],[4.33-bmj,2.605+bev],[4.33+bmj,2.605+bmi]];
	}
	
	coords = [
		[upper_x+rot_adj[0][0], upper_y+rot_adj[0][1]],
		[upper_x+4.33+rot_adj[1][0], upper_y-2.605+rot_adj[1][1]],
		[upper_x+rot_adj[2][0], upper_y-5.21+rot_adj[2][1]],
		[upper_x-4.33+rot_adj[3][0], upper_y-2.605+rot_adj[3][1]],
		[upper_x+rot_adj_base[0][0], upper_y+rot_adj_base[0][1]],
		[upper_x+4.33+rot_adj_base[1][0], upper_y-2.605+rot_adj_base[1][1]],
		[upper_x+rot_adj_base[2][0], upper_y-5.21+rot_adj_base[2][1]],
		[upper_x-4.33+rot_adj_base[3][0], upper_y-2.605+rot_adj_base[3][1]]
	];

	for(i=0;i<4;i++) {
		geometry.vertices.push( new THREE.Vector3 (coords[0][0] - geocenter_x,  hang_dist, coords[0][1] - geocenter_z));
		geometry.vertices.push( new THREE.Vector3 (coords[1][0] - geocenter_x,  hang_dist, coords[1][1] - geocenter_z));
		geometry.vertices.push( new THREE.Vector3 (coords[2][0] - geocenter_x,  hang_dist, coords[2][1] - geocenter_z));
		geometry.vertices.push( new THREE.Vector3 (coords[3][0] - geocenter_x,  hang_dist, coords[3][1] - geocenter_z));
		geometry.vertices.push( new THREE.Vector3 ((coords[1][0] + coords[3][0])/2 - geocenter_x,  0, (coords[1][1] + coords[3][1])/2 - geocenter_z));

		geometry.faces.push( new THREE.Face3(0, 1, 4) );
		geometry.faces.push( new THREE.Face3(1, 2, 4) );
		geometry.faces.push( new THREE.Face3(2, 3, 4) );
		geometry.faces.push( new THREE.Face3(3, 0, 4) );

		//four corners
		geometry.vertices.push( new THREE.Vector3 (coords[4][0] - geocenter_x,  0, coords[4][1] - geocenter_z));
		geometry.vertices.push( new THREE.Vector3 (coords[5][0] - geocenter_x,  0, coords[5][1] - geocenter_z));
		geometry.vertices.push( new THREE.Vector3 (coords[6][0] - geocenter_x,  0, coords[6][1] - geocenter_z));
		geometry.vertices.push( new THREE.Vector3 (coords[7][0] - geocenter_x,  0, coords[7][1] - geocenter_z));

		//midpoints
		geometry.vertices.push( new THREE.Vector3 (((coords[4][0] + coords[5][0])/2.0) - geocenter_x,  3, ((coords[0][1] + coords[1][1])/2.0) - geocenter_z));
		geometry.vertices.push( new THREE.Vector3 (((coords[5][0] + coords[6][0])/2.0) - geocenter_x,  3, ((coords[1][1] + coords[2][1])/2.0) - geocenter_z));
		geometry.vertices.push( new THREE.Vector3 (((coords[6][0] + coords[7][0])/2.0) - geocenter_x,  3, ((coords[2][1] + coords[3][1])/2.0) - geocenter_z));
		geometry.vertices.push( new THREE.Vector3 (((coords[7][0] + coords[4][0])/2.0) - geocenter_x,  3, ((coords[3][1] + coords[0][1])/2.0) - geocenter_z));

		//geometry.faces.push( new THREE.Face3(5, 6, 7) );
		//geometry.faces.push( new THREE.Face3(5, 7, 8) );

		geometry.faces.push( new THREE.Face3(0, 1, 5) );
		geometry.faces.push( new THREE.Face3(6, 5, 1) );
			
		geometry.faces.push( new THREE.Face3(1, 2, 6) );
		geometry.faces.push( new THREE.Face3(7, 6, 2) );
		
		geometry.faces.push( new THREE.Face3(2, 3, 7) );
		geometry.faces.push( new THREE.Face3(8, 7, 3) );
		
		geometry.faces.push( new THREE.Face3(3, 0, 8) );
		geometry.faces.push( new THREE.Face3(5, 8, 0) );						
	}
	
	//add UV coordinates for texturing
	for (i=0; i<4; i++) {
	  geometry.faceVertexUvs[0].push([
		  new THREE.Vector2(0,0),
		  new THREE.Vector2(1,0),
		  new THREE.Vector2(.5,1)
	  ]);
	}
	for (i=0; i<8; i++) {
	  geometry.faceVertexUvs[0].push([
		  new THREE.Vector2(0,0),
		  new THREE.Vector2(1,0),
		  new THREE.Vector2(.2,1),
		  new THREE.Vector2(0,.2),
		  new THREE.Vector2(1,.2),
		  new THREE.Vector2(1,0),
	  ]);
	}
	
	return geometry;
}

function Syd_QT_Feature(data) {
    this.box_count = 0;
    this.boxes = [];

    for (var i=0; i<data.length; i++) {
		this.boxes.push(new Syd_QT_Box(
			new Syd_QT_Point(data[i][0][0],data[i][0][2],data[i][0][1]),
			new Syd_QT_Point(data[i][1][0],data[i][1][2],data[i][1][1]),
			new Syd_QT_Point(data[i][2][0],data[i][2][2],data[i][2][1]),
			new Syd_QT_Point(data[i][3][0],data[i][3][2],data[i][3][1]),
			new Syd_QT_Point(data[i][4][0],data[i][4][2],data[i][4][1]),
			new Syd_QT_Point(data[i][5][0],data[i][5][2],data[i][5][1]),
			new Syd_QT_Point(data[i][6][0],data[i][6][2],data[i][6][1]),
			new Syd_QT_Point(data[i][7][0],data[i][7][2],data[i][7][1])
		));
		this.box_count++;
    }
    this.getBox = function(indx) {
    	return this.boxes[indx];
    }
    this.getBoxCount = function() {
    	return this.box_count;
    }
};

function Syd_QT_Box(ul1, ll1, lr1, ur1, ul2, ll2, lr2, ur2) {
    this.ul1 = ul1;
    this.ll1 = ll1;
    this.lr1 = lr1;
    this.ur1 = ur1;
    this.ul2 = ul2;
    this.ll2 = ll2;
    this.lr2 = lr2;
    this.ur2 = ur2;

    this.getFloats = function() {
    var flts = [];

    var pt = this.ul1.getValues();

    flts.push(pt[0]);
    flts.push(pt[1]);
    flts.push(pt[2]);
    pt = this.ur1.getValues();
    flts.push(pt[0]);
    flts.push(pt[1]);
    flts.push(pt[2]);
    pt = this.lr1.getValues();
    flts.push(pt[0]);
    flts.push(pt[1]);
    flts.push(pt[2]);
    pt = this.ll1.getValues();
    flts.push(pt[0]);
    flts.push(pt[1]);
    flts.push(pt[2]);
    pt = this.ul2.getValues();
    flts.push(pt[0]);
    flts.push(pt[1]);
    flts.push(pt[2]);
    pt = this.ur2.getValues();
    flts.push(pt[0]);
    flts.push(pt[1]);
    flts.push(pt[2]);
    pt = this.lr2.getValues();
    flts.push(pt[0]);
    flts.push(pt[1]);
    flts.push(pt[2]);
    pt = this.ll2.getValues();
    flts.push(pt[0]);
    flts.push(pt[1]);
    flts.push(pt[2]);

    return flts;
    },
    this.getFloat = function(indx) {
    switch(indx) {
        case 0: return this.ul1.getX();
        case 1: return this.ul1.getY();
        case 2: return this.ul1.getZ();
        case 3: return this.ur1.getX();
        case 4: return this.ur1.getY();
        case 5: return this.ur1.getZ();
        case 6: return this.lr1.getX();
        case 7: return this.lr1.getY();
        case 8: return this.lr1.getZ();
        case 9: return this.ll1.getX();
        case 10: return this.ll1.getY();
        case 11: return this.ll1.getZ();
        case 12: return this.ul2.getX();
        case 13: return this.ul2.getY();
        case 14: return this.ul2.getZ();
        case 15: return this.ur2.getX();
        case 16: return this.ur2.getY();
        case 17: return this.ur2.getZ();
        case 18: return this.lr2.getX();
        case 19: return this.lr2.getY();
        case 20: return this.lr2.getZ();
        case 21: return this.ll2.getX();
        case 22: return this.ll2.getY();
        case 23: return this.ll2.getZ();
        default: false
    }
    },
    this.setFloat = function(indx, value) {
    switch(indx) {
        case 0: this.ul1.setX(value);
        case 1: this.ul1.setY(value);
        case 2: this.ul1.setZ(value);
        case 3: this.ur1.setX(value);
        case 4: this.ur1.setY(value);
        case 5: this.ur1.setZ(value);
        case 6: this.lr1.setX(value);
        case 7: this.lr1.setY(value);
        case 8: this.lr1.setZ(value);
        case 9: this.ll1.setX(value);
        case 10: this.ll1.setY(value);
        case 11: this.ll1.setZ(value);
        case 12: this.ul2.setX(value);
        case 13: this.ul2.setY(value);
        case 14: this.ul2.setZ(value);
        case 15: this.ur2.setX(value);
        case 16: this.ur2.setY(value);
        case 17: this.ur2.setZ(value);
        case 18: this.lr2.setX(value);
        case 19: this.lr2.setY(value);
        case 20: this.lr2.setZ(value);
        case 21: this.ll2.setX(value);
        case 22: this.ll2.setY(value);
        case 23: this.ll2.setZ(value);
        default: false
    }
    }
};

function Syd_QT_Point(x, y, z) {
    this.x = x;
    this.y = y;
    this.z = z;

    this.getValues = function() {
    var flts = [];

    flts.push(this.x);
    flts.push(this.y);
    flts.push(this.z);

    return flts;
    },
    this.getX = function() {
    return this.x;
    },
    this.getY = function() {
    return this.y;
    },
    this.getZ = function() {
    return this.z;
    },
    this.setX = function(value) {
    this.x = value;
    },
    this.setY = function(value) {
    this.y = value;
    },
    this.setZ = function(value) {
    this.z = value;
    }
};

function initializeFeatureOrientation(height) {
    targetPitch = 0.0;
    group.position.y = 0.0;
    group.rotation.z = 0.0;
    group.position.z = 0.0;
    if(featureType < 2) { //Partitions
        if(featureType == 0) {
            targetRotation = Math.PI/6;
        } else { //featureType 1
            targetRotation = -Math.PI;
        }
    } else if(featureType == 2) { //Flat Walls
        targetRotation = Math.PI/6;
	} else if(featureType == 4) { //Flat Ceiling
        targetPitch  = Math.PI/2.5;
		//targetYaw = Math.PI/20;
    } else if(featureType == 3) { //Wall-Ceiling
        targetRotation = Math.PI/3;
        group.position.y = featureSize.y;
    } else if(featureType == 5) { //Ceiling
        targetRotation = Math.PI/5;
        group.position.y = featureSize.y;
    } else if (featureType == 400) {
    	targetRotation = 0.0;
    	targetPitch  = Math.PI/2;
    	group.position.y = scene_height/2;
    } else if (featureType == 500) {
    	targetRotation = 0.0;
    	targetPitch  = Math.PI/2;
    } else if (featureType > 99) {
    	targetRotation = 0.0;
    	targetPitch  = 6/8*Math.PI;
    	group.position.y = scene_height/2;
    }
};

function assignUVs(geometry) {
	faceCount = 0;
    geometry.faceVertexUvs[0] = [];
    if(featureType < 99 || featureType == 400) {
		geometry.faces.forEach(function(face) {
			var components = ['x', 'y', 'z'].sort(function(a, b) {
				return Math.abs(face.normal[a]) > Math.abs(face.normal[b]);
			});

			var v1 = geometry.vertices[face.a];
			var v2 = geometry.vertices[face.b];
			var v3 = geometry.vertices[face.c];

			//alert(v1[components[0]] + "," + v1[components[1]] + "\n" + 
			//      v2[components[0]] + "," + v2[components[1]] + "\n" + 
			//      v3[components[0]] + "," + v3[components[1]]);

			geometry.faceVertexUvs[0].push([
				new THREE.Vector2(v1[components[0]], v1[components[1]]),
				new THREE.Vector2(v2[components[0]], v2[components[1]]),
				new THREE.Vector2(v3[components[0]], v3[components[1]])
			]);
			faceCount++;
		});
    } else if (featureType==100) {
		geometry.faceVertexUvs[0].push([ //1	
			new THREE.Vector2(0,0),
			new THREE.Vector2(1,0),
			new THREE.Vector2(1,1)
		]);  		
		geometry.faceVertexUvs[0].push([ //2	
			new THREE.Vector2(0,1),
			new THREE.Vector2(0,0),
			new THREE.Vector2(0,0)
		]);
		geometry.faceVertexUvs[0].push([ //3	
			new THREE.Vector2(1,1),
			new THREE.Vector2(1,0),
			new THREE.Vector2(0,0)
		]);
		geometry.faceVertexUvs[0].push([ //4	
			new THREE.Vector2(1,1),
			new THREE.Vector2(0,0),
			new THREE.Vector2(0,1)
		]);  		
		geometry.faceVertexUvs[0].push([ //5	
			new THREE.Vector2(0,1),
			new THREE.Vector2(0,0),
			new THREE.Vector2(0.15,1)
		]);  		
		geometry.faceVertexUvs[0].push([ //6
			new THREE.Vector2(0,0),
			new THREE.Vector2(0.013,0),
			new THREE.Vector2(0.15,1)
		]);
		geometry.faceVertexUvs[0].push([ //7
			new THREE.Vector2(0,0),
			new THREE.Vector2(0,1),
			new THREE.Vector2(0.013,0)
		]);  		
		geometry.faceVertexUvs[0].push([ //8
			new THREE.Vector2(0,1),
			new THREE.Vector2(0.15,1),
			new THREE.Vector2(0.013,0)
		]);
		geometry.faceVertexUvs[0].push([ //9
			new THREE.Vector2(1,0),
			new THREE.Vector2(0,0),
			new THREE.Vector2(1,0.013)
		]);  		
		geometry.faceVertexUvs[0].push([ //10
			new THREE.Vector2(0,0),
			new THREE.Vector2(0,0.013),
			new THREE.Vector2(1,0.013)
		]);
		geometry.faceVertexUvs[0].push([ //11
			new THREE.Vector2(0,0),
			new THREE.Vector2(1,0),
			new THREE.Vector2(0,0.15)
		]);  		
		geometry.faceVertexUvs[0].push([ //12
			new THREE.Vector2(1,0),
			new THREE.Vector2(1,0.15),
			new THREE.Vector2(0,0.15)
		]);
    } else if (featureType==200) {	
		//add any additional vertices
		if(featureType==200) {
			geometry.faces.forEach(function(face) {
				if(faceCount%2==0) {
					geometry.faceVertexUvs[0].push([ //1	
						new THREE.Vector2(0,0),
						new THREE.Vector2(1,0),
						new THREE.Vector2(1,1)
					]);					
				} else {
					geometry.faceVertexUvs[0].push([ //2	
						new THREE.Vector2(1,0),
						new THREE.Vector2(0,1),
						new THREE.Vector2(0,0)
					]);
				}
				faceCount++;
			});
		}
    }
    geometry.uvsNeedUpdate = true;
}

function onWindowResize() {
/*
    windowHalfX = window.innerWidth / 2;
    windowHalfY = window.innerHeight / 2;
    camera.aspect = window.innerWidth / window.innerHeight;
    camera.updateProjectionMatrix();
    renderer.setSize( window.innerWidth, window.innerHeight );
*/
}
//
function onDocumentMouseDown( event ) {
    event.preventDefault();

    // HEAD's UP DISPLAY HERE
    // find intersections
    var mouse = new THREE.Vector2();

    var rect = document.getElementById('visualization').getBoundingClientRect();
    //get mouse values as proportional screen location from 0 to 1    
    mouse.x = ( (event.clientX - rect.left) / renderer.domElement.clientWidth ) * 2 - 1;
    mouse.y = - ( (event.clientY - rect.top) / renderer.domElement.clientHeight ) * 2 + 1;

    var raycaster = new THREE.Raycaster();
    raycaster.setFromCamera( mouse, camera );

    var intersects = raycaster.intersectObjects( objects );

    //alert(mouse.x + "," + mouse.y);
    //map mouse clicks to HUD interaction handlers
    if ( intersects.length > 0 ) {
		processHUDClick(mouse.x, mouse.y);
        /* //test for intersection visually
        var loader = new THREE.TextureLoader();
        var texture = loader.load( material );
        var resetMaterial = new THREE.SpriteMaterial( { map: texture, color: 0xff0000, transparent: true, opacity: 0.50  } );
        var particle = new THREE.Sprite( resetMaterial );
        particle.position.copy( intersects[ 0 ].point );
        particle.scale.x = particle.scale.y = .1;
        scene.add( particle );
        */
    }
    //console.log("Intersected objects: " + intersects);

    var event = event || window.event;
    if ('object' === typeof event) {
        btnCode = event.button;

        switch (btnCode) {
            case 0:
                //console.log('Left button clicked.');
            break;

            case 1:
                //console.log('Middle button clicked.');
            break;

            case 2:
                //console.log('Right button clicked.');
            break;

            default:
                //console.log('Unexpected code: ' + btnCode);
        }
    }

    document.getElementById('visualization').addEventListener( 'mousemove', onDocumentMouseMove, false );
    document.getElementById('visualization').addEventListener( 'mouseup', onDocumentMouseUp, false );
    document.getElementById('visualization').addEventListener( 'mouseout', onDocumentMouseOut, false );
    mouseXOnMouseDown = event.clientX - windowHalfX;
    targetRotationOnMouseDown = targetRotation;
}
function onDocumentMouseMove( event ) {

    switch (btnCode) {

    case 1: //Middle button
        if(prevMouseX == -1) {
            prevMouseX = event.clientX;
            mouseX = event.clientX;
        } else {
            prevMouseX = mouseX;
            mouseX = event.clientX;
            targetPanX = (mouseX - prevMouseX) * 0.10;
        }
        if(prevMouseY == -1) {
            prevMouseY = event.clientY;
            mouseY = event.clientY;
        } else {
            prevMouseY = mouseY;
            mouseY = event.clientY;
            targetPanY = (prevMouseY - mouseY) * 0.10;
        }
    break;
    case 2: //Right button
        if(prevMouseY == -1) {
            prevMouseY = event.clientY;
            mouseY = event.clientY;
        } else {
            prevMouseY = mouseY;
            mouseY = event.clientY;
            targetTranslation += (prevMouseY - mouseY) * 0.10;
        }
    break;
    case 0:  //Left button
    default: //or some other unexpected button event
        mouseX = event.clientX - windowHalfX;
        if(prevMouseY == -1) {
            prevMouseY = event.clientY;
            mouseY = event.clientY;
        } else {
            prevMouseY = mouseY;
            mouseY = event.clientY;
            targetPitch += (mouseY - prevMouseY) * 0.002;
        }
        targetRotation = targetRotationOnMouseDown + ( mouseX - mouseXOnMouseDown ) * 0.002;
    }
}
function onDocumentMouseUp( event ) {
    document.getElementById('visualization').removeEventListener( 'mousemove', onDocumentMouseMove, false );
    document.getElementById('visualization').removeEventListener( 'mouseup', onDocumentMouseUp, false );
    document.getElementById('visualization').removeEventListener( 'mouseout', onDocumentMouseOut, false );
    prevMouseY = -1; prevMouseX = -1; mouseX = -1; mouseY = -1; targetPanX = 0; targetPanY = 0;
}
function onDocumentMouseOut( event ) {
    document.getElementById('visualization').removeEventListener( 'mousemove', onDocumentMouseMove, false );
    document.getElementById('visualization').removeEventListener( 'mouseup', onDocumentMouseUp, false );
    document.getElementById('visualization').removeEventListener( 'mouseout', onDocumentMouseOut, false );
}
function onMouseWheel( event ) {
    var delta = Math.max(-1, Math.min(1, (event.wheelDelta || -event.detail)));
    targetTranslation += delta * 5.0;
}
function onDocumentTouchStart( event ) {
    if ( event.touches.length == 1 ) {
        event.preventDefault();
        // HEAD's UP DISPLAY HERE
        // find intersections
        var mouse = new THREE.Vector2();

        var rect = document.getElementById('visualization').getBoundingClientRect();    
        mouse.x = ( (event.touches[ 0 ].pageX - rect.left) / renderer.domElement.clientWidth ) * 2 - 1;
        mouse.y = - ( (event.touches[ 0 ].pageY - rect.top) / renderer.domElement.clientHeight ) * 2 + 1;

        var raycaster = new THREE.Raycaster();
        raycaster.setFromCamera( mouse, camera );

        var intersects = raycaster.intersectObjects( objects );

        //map mouse clicks to HUD interaction handlers
        if ( intersects.length > 0 ) {
			processHUDClick(mouse.x, mouse.y);
        }
        mouseXOnMouseDown = event.touches[ 0 ].pageX - windowHalfX;
        targetRotationOnMouseDown = targetRotation;
    }
}

function processHUDClick(x,y) {
	//alert(x + "," + y);
	if(y > .86) {
		resetView();
	} else if(y < -.84) {
		if(x < .82) {
			zoomIn();
		} else {
			zoomOut();
		}
	} else if(y > -.58) {
		pitchUp();
	} else if(y < -.7) {
		pitchDown();
	} else {
		if(x < .82) {
			yawLeft();
		} else {
			yawRight();
		}
	}
}

function onDocumentTouchMove( event ) {
    if ( event.touches.length == 1 ) {
        event.preventDefault();
        mouseX = event.touches[ 0 ].pageX - windowHalfX;
        if(prevMouseY == -1) {
            prevMouseY = event.touches[ 0 ].pageY;
            mouseY = event.touches[ 0 ].pageY;
        } else {
            prevMouseY = mouseY;
            mouseY = event.touches[ 0 ].pageY;
            targetPitch += (mouseY - prevMouseY) * 0.002;
        }
        targetRotation = targetRotationOnMouseDown + ( mouseX - mouseXOnMouseDown ) * 0.002;
    }
}
//
function animate() {
    requestAnimationFrame( animate );
    render();
}
function render() {
    group.rotation.x += ( targetPitch - group.rotation.x );
    group.rotation.y += ( targetRotation - group.rotation.y ) * 0.05;
	group.rotation.z += ( targetYaw - group.rotation.z ) * 0.05;
    group.position.x += ( targetPanX );
    if(featureType == 400) {
    	group.position.z += ( targetPanY );
    	group.position.y += ( targetTranslation );
    } else {
    	group.position.y += ( targetPanY );
    	group.position.z += ( targetTranslation );
    }
    targetTranslation = 0;

    // Render the Seeyond scene.
    renderer.clear();
    renderer.render( scene, camera );
}
function resetView() {
    targetPitch = 0;
	targetYaw = 0;
    targetRotation = 0;
    targetTranslation = 0;
    targetPanX = 0;
    targetPanY = 0;
    targetRotationOnMouseDown = 0;
    mouseX = 0;
    mouseY = -1;
    prevMouseX = -1;
    prevMouseY = -1;
    mouseXOnMouseDown = 0;
    initializeFeatureOrientation(100);
}
function zoomIn() {
    targetTranslation += 10.0;
}
function zoomOut() {
    targetTranslation -= 10.0;
}
function pitchUp() {
    targetPitch += .03;
}
function pitchDown() {
    targetPitch -= .03;
}
function yawRight() {
    targetRotation += .03;
}
function yawLeft() {
    targetRotation -= .03;
}
function OrientX(surf, g_x, g_y, g_z) {
	//rotate the feature so feature horizontal axis aligns with x axis
	var base_edge = [];
	if(surf=="front") {
		base_edge = [
			(leftBox[0]-g_x) - (rightBox[9]-g_x),
			(leftBox[1]-g_y) - (rightBox[10]-g_y), 
			(leftBox[2]-g_z) - (rightBox[11]-g_z) 
		];
	} else { //use the back surface for orientation
		base_edge = [
			(leftBox[12]-g_x) - (rightBox[21]-g_x),
			(leftBox[13]-g_y) - (rightBox[22]-g_y),
			(leftBox[14]-g_z) - (rightBox[23]-g_z)
		];
	}
	var vector1 = normalize(base_edge);
	var vector2 = [1.0, 0.0, 0.0];
	
	//If unit_normal and xy_normal are normalized so that |unit_normal|=|xy_normal|=1, then:
	var angle = Math.acos(vector1[0]*vector2[0] + vector1[1]*vector2[1] + vector1[2]*vector2[2]);

	var rq = new THREE.Quaternion();
	rq.setFromAxisAngle(new THREE.Vector3(0, 1, 0), angle);
	group.setRotationFromQuaternion(rq);
}
function normalize(v) {
	length = Math.sqrt(v[0]*v[0] + v[1]*v[1] + v[2]*v[2]);
	v[0] = v[0]/length;
	v[1] = v[1]/length;
	v[2] = v[2]/length;
	return v;
}